@@ -8,14 +8,13 @@
  *******************************************************************************/
 package org.eclipse.egit.core.synchronize;
 
-import static org.eclipse.compare.structuremergeviewer.Differencer.ADDITION;
-import static org.eclipse.compare.structuremergeviewer.Differencer.CHANGE;
-import static org.eclipse.compare.structuremergeviewer.Differencer.DELETION;
-import static org.eclipse.compare.structuremergeviewer.Differencer.LEFT;
-import static org.eclipse.compare.structuremergeviewer.Differencer.RIGHT;
+import static org.eclipse.egit.core.synchronize.GitCommitsModelCache.ADDITION;
+import static org.eclipse.egit.core.synchronize.GitCommitsModelCache.CHANGE;
+import static org.eclipse.egit.core.synchronize.GitCommitsModelCache.DELETION;
+import static org.eclipse.egit.core.synchronize.GitCommitsModelCache.LEFT;
+import static org.eclipse.egit.core.synchronize.GitCommitsModelCache.RIGHT;
 import static org.eclipse.jgit.junit.JGitTestUtil.deleteTrashFile;
 import static org.eclipse.jgit.junit.JGitTestUtil.writeTrashFile;
-import static org.eclipse.jgit.lib.ObjectId.zeroId;
 import static org.hamcrest.CoreMatchers.is;
 import static org.hamcrest.CoreMatchers.not;
 import static org.hamcrest.CoreMatchers.notNullValue;
@@ -29,92 +28,36 @@ import org.eclipse.egit.core.synchronize.GitCommitsModelCache.Change;
 import org.eclipse.egit.core.synchronize.GitCommitsModelCache.Commit;
 import org.eclipse.jgit.api.Git;
 import org.eclipse.jgit.errors.AmbiguousObjectException;
-import org.eclipse.jgit.junit.LocalDiskRepositoryTestCase;
-import org.eclipse.jgit.lib.AbbreviatedObjectId;
 import org.eclipse.jgit.lib.ObjectId;
 import org.eclipse.jgit.revwalk.RevCommit;
-import org.eclipse.jgit.storage.file.FileRepository;
-import org.junit.Before;
+import org.eclipse.jgit.treewalk.filter.PathFilter;
 import org.junit.Test;
 
 @SuppressWarnings("boxing")
-public class GitCommitsModelCacheTest extends LocalDiskRepositoryTestCase {
-
-	private FileRepository db;
-
-	private static final String INITIAL_TAG = "initial-tag";
-
-	private static final AbbreviatedObjectId ZERO_ID = AbbreviatedObjectId
-			.fromObjectId(zeroId());
-
-	@Before
-	@Override
-	// copied from org.eclipse.jgit.lib.RepositoryTestCase
-	public void setUp() throws Exception {
-		super.setUp();
-		db = createWorkRepository();
-		Git git = new Git(db);
-		git.commit().setMessage("initial commit").call();
-		git.tag().setName(INITIAL_TAG).call();
-	}
-
+public class GitCommitsModelCacheTest extends AbstractCacheTest {
 	@Test
 	public void shouldReturnEmptyListForSameSrcAndDstCommit() throws Exception {
 		// given
 		Git git = new Git(db);
 		RevCommit c = commit(git, "second commit");
-
 		// when
-		List<Commit> result = GitCommitsModelCache.build(db, c, c);
-
+		List<Commit> result = GitCommitsModelCache.build(db, c, c, null);
 		// then
 		assertThat(result, notNullValue());
 		assertThat(result.size(), is(0));
 	}
 
 	@Test
-	public void shouldListOneEmptyCommit() throws Exception {
+	public void shouldNotListEmptyCommits() throws Exception {
 		// given
 		Git git = new Git(db);
 		RevCommit c = commit(git, "second commit");
-
 		// when
-		List<Commit> leftResult = GitCommitsModelCache.build(db, initialTagId(), c);
-		List<Commit> rightResult = GitCommitsModelCache.build(db, c, initialTagId());
-
+		List<Commit> result = GitCommitsModelCache.build(db, initialTagId(), c,
+				null);
 		// then
-		// left assertions
-		assertThat(leftResult, notNullValue());
-		assertThat(leftResult.size(), is(1));
-		assertEmptyCommit(leftResult.get(0), c, LEFT);
-
-		// right assertions
-		assertThat(rightResult, notNullValue());
-		assertThat(rightResult.size(), is(1));
-		assertEmptyCommit(rightResult.get(0), c, RIGHT);
-	}
-
-	@Test
-	public void shouldListTwoEmptyCommits() throws Exception {
-		// given
-		Git git = new Git(db);
-		RevCommit c1 = commit(git, "second commit");
-		RevCommit c2 = commit(git, "third commit");
-
-		// when
-		List<Commit> leftResult = GitCommitsModelCache.build(db, initialTagId(), c2);
-		List<Commit> rightResult = GitCommitsModelCache.build(db, c2, initialTagId());
-
-		// then
-		assertThat(leftResult, notNullValue());
-		assertThat(leftResult.size(), is(2));
-		assertEmptyCommit(leftResult.get(0), c2, LEFT);
-		assertEmptyCommit(leftResult.get(1), c1, LEFT);
-
-		assertThat(rightResult, notNullValue());
-		assertThat(rightResult.size(), is(2));
-		assertEmptyCommit(rightResult.get(0), c2, RIGHT);
-		assertEmptyCommit(rightResult.get(1), c1, RIGHT);
+		assertThat(result, notNullValue());
+		assertThat(result.size(), is(0));
 	}
 
 	@Test
@@ -124,51 +67,98 @@ public class GitCommitsModelCacheTest extends LocalDiskRepositoryTestCase {
 		writeTrashFile(db, "a.txt", "content");
 		git.add().addFilepattern("a.txt").call();
 		RevCommit c = commit(git, "first commit");
-
 		// when
-		List<Commit> leftResult = GitCommitsModelCache.build(db, initialTagId(), c);
-		List<Commit> rightResult = GitCommitsModelCache.build(db, c, initialTagId());
-
+		List<Commit> leftResult = GitCommitsModelCache.build(db,
+				initialTagId(), c, null);
+		List<Commit> rightResult = GitCommitsModelCache.build(db, c,
+				initialTagId(), null);
 		// then
 		// left assertions
 		assertThat(leftResult, notNullValue());
 		assertCommit(leftResult.get(0), c, 1);
-		assertFileDeletion(c, leftResult.get(0).getChildren().get("a.txt"), "a.txt", LEFT | DELETION);
-
-		// right assertions, after changing sides addition becomes deletion
+		assertFileDeletion(c, leftResult.get(0).getChildren().get("a.txt"),
+				"a.txt", LEFT);
+		// right asserts, after changing sides addition becomes deletion
 		assertThat(rightResult, notNullValue());
 		assertCommit(rightResult.get(0), c, 1);
-		assertFileAddition(c, rightResult.get(0).getChildren().get("a.txt"), "a.txt", RIGHT | ADDITION);
+		assertFileAddition(c, rightResult.get(0).getChildren().get("a.txt"),
+				"a.txt", RIGHT);
 	}
 
 	@Test
-	public void shouldListAdditionOrDeletionInsideFolderInCommit() throws Exception {
+	public void shouldListAdditionOrDeletionInsideFolderInCommit()
+			throws Exception {
 		// given
 		Git git = new Git(db);
 		writeTrashFile(db, "folder/a.txt", "content");
 		git.add().addFilepattern("folder/a.txt").call();
 		RevCommit c = commit(git, "first commit");
-
 		// when
-		List<Commit> leftResult = GitCommitsModelCache.build(db, initialTagId(), c);
-		List<Commit> rightResult = GitCommitsModelCache.build(db, c, initialTagId());
-
+		List<Commit> leftResult = GitCommitsModelCache.build(db,
+				initialTagId(), c, null);
+		List<Commit> rightResult = GitCommitsModelCache.build(db, c,
+				initialTagId(), null);
 		// then
 		// left assertions
 		assertThat(leftResult, notNullValue());
 		assertCommit(leftResult.get(0), c, 1);
 		assertThat(leftResult.get(0).getChildren().size(), is(1));
-		assertFileDeletion(c, leftResult.get(0).getChildren().get("folder/a.txt"), "a.txt", LEFT | DELETION);
-
-		// right assertions, after changing sides addition becomes deletion
+		assertFileDeletion(c,
+				leftResult.get(0).getChildren().get("folder/a.txt"), "a.txt",
+				LEFT);
+		// right asserts, after changing sides addition becomes deletion
 		assertThat(rightResult, notNullValue());
 		assertCommit(rightResult.get(0), c, 1);
 		assertThat(rightResult.get(0).getChildren().size(), is(1));
-		assertFileAddition(c, rightResult.get(0).getChildren().get("folder/a.txt"), "a.txt", RIGHT | ADDITION);
+		assertFileAddition(c,
+				rightResult.get(0).getChildren().get("folder/a.txt"), "a.txt",
+				RIGHT);
 	}
 
 	@Test
-	public void shouldListAdditionsOrDeletionsInsideSeparateFoldersInCommit() throws Exception {
+	public void shouldListAdditionsOrDeletionsInsideSeparateFoldersInCommit()
+			throws Exception {
+		// given
+		Git git = new Git(db);
+		writeTrashFile(db, "folder/a.txt", "content");
+		writeTrashFile(db, "folder2/b.txt", "b content");
+		git.add().addFilepattern("folder/a.txt").call();
+		git.add().addFilepattern("folder2/b.txt").call();
+		RevCommit c = commit(git, "first commit");
+		// when
+		List<Commit> leftResult = GitCommitsModelCache.build(db,
+				initialTagId(), c, null);
+		List<Commit> rightResult = GitCommitsModelCache.build(db, c,
+				initialTagId(), null);
+		// then
+		// left assertions
+		assertThat(leftResult, notNullValue());
+		assertThat(Integer.valueOf(leftResult.size()), is(Integer.valueOf(1)));
+		assertThat(leftResult.get(0).getShortMessage(), is("first commit"));
+		assertThat(leftResult.get(0).getChildren(), notNullValue());
+		assertThat(leftResult.get(0).getChildren().size(), is(2));
+		assertFileDeletion(c,
+				leftResult.get(0).getChildren().get("folder/a.txt"), "a.txt",
+				LEFT);
+		assertFileDeletion(c,
+				leftResult.get(0).getChildren().get("folder2/b.txt"), "b.txt",
+				LEFT);
+		// right asserts, after changing sides addition becomes deletion
+		assertThat(rightResult, notNullValue());
+		assertThat(Integer.valueOf(rightResult.size()), is(Integer.valueOf(1)));
+		assertThat(rightResult.get(0).getShortMessage(), is("first commit"));
+		assertThat(rightResult.get(0).getChildren(), notNullValue());
+		assertThat(rightResult.get(0).getChildren().size(), is(2));
+		assertFileAddition(c,
+				rightResult.get(0).getChildren().get("folder/a.txt"), "a.txt",
+				RIGHT);
+		assertFileAddition(c,
+				rightResult.get(0).getChildren().get("folder2/b.txt"), "b.txt",
+				RIGHT);
+	}
+
+	@Test
+	public void shouldApplyPathFilter() throws Exception {
 		// given
 		Git git = new Git(db);
 		writeTrashFile(db, "folder/a.txt", "content");
@@ -178,66 +168,57 @@ public class GitCommitsModelCacheTest extends LocalDiskRepositoryTestCase {
 		RevCommit c = commit(git, "first commit");
 
 		// when
-		List<Commit> leftResult = GitCommitsModelCache.build(db, initialTagId(), c);
-		List<Commit> rightResult = GitCommitsModelCache.build(db, c, initialTagId());
-
+		PathFilter pathFilter = PathFilter.create("folder");
+		List<Commit> leftResult = GitCommitsModelCache.build(db,
+				initialTagId(), c, pathFilter);
 		// then
-		// left assertions
 		assertThat(leftResult, notNullValue());
 		assertThat(Integer.valueOf(leftResult.size()), is(Integer.valueOf(1)));
 		assertThat(leftResult.get(0).getShortMessage(), is("first commit"));
-
 		assertThat(leftResult.get(0).getChildren(), notNullValue());
-		assertThat(leftResult.get(0).getChildren().size(), is(2));
-
-		assertFileDeletion(c, leftResult.get(0).getChildren().get("folder/a.txt"), "a.txt", LEFT | DELETION);
-		assertFileDeletion(c, leftResult.get(0).getChildren().get("folder2/b.txt"), "b.txt",LEFT | DELETION);
-
-		// right assertions, after changing sides addition becomes deletion
-		assertThat(rightResult, notNullValue());
-		assertThat(Integer.valueOf(rightResult.size()), is(Integer.valueOf(1)));
-		assertThat(rightResult.get(0).getShortMessage(), is("first commit"));
-
-		assertThat(rightResult.get(0).getChildren(), notNullValue());
-		assertThat(rightResult.get(0).getChildren().size(), is(2));
-
-		assertFileAddition(c, rightResult.get(0).getChildren().get("folder/a.txt"), "a.txt", RIGHT | ADDITION);
-		assertFileAddition(c, rightResult.get(0).getChildren().get("folder2/b.txt"), "b.txt",RIGHT | ADDITION);
+		assertThat(leftResult.get(0).getChildren().size(), is(1));
+		assertFileDeletion(c,
+				leftResult.get(0).getChildren().get("folder/a.txt"), "a.txt",
+				LEFT);
 	}
 
 	@Test
-	public void shouldListAdditionsOrDeletionsInsideFolderInCommit() throws Exception {
+	public void shouldListAdditionsOrDeletionsInsideFolderInCommit()
+			throws Exception {
 		// given
 		Git git = new Git(db);
 		writeTrashFile(db, "folder/a.txt", "content");
 		writeTrashFile(db, "folder/b.txt", "b content");
 		git.add().addFilepattern("folder").call();
 		RevCommit c = commit(git, "first commit");
-
 		// when
-		List<Commit> leftResult = GitCommitsModelCache.build(db, initialTagId(), c);
-		List<Commit> rightResult = GitCommitsModelCache.build(db, c, initialTagId());
-
+		List<Commit> leftResult = GitCommitsModelCache.build(db,
+				initialTagId(), c, null);
+		List<Commit> rightResult = GitCommitsModelCache.build(db, c,
+				initialTagId(), null);
 		// then
 		// left assertions
 		assertThat(leftResult, notNullValue());
 		assertThat(Integer.valueOf(leftResult.size()), is(Integer.valueOf(1)));
 		assertCommit(leftResult.get(0), c, 2);
-
 		assertThat(leftResult.get(0).getChildren().size(), is(2));
-
-		assertFileDeletion(c, leftResult.get(0).getChildren().get("folder/a.txt"), "a.txt", LEFT | DELETION);
-		assertFileDeletion(c, leftResult.get(0).getChildren().get("folder/b.txt"), "b.txt", LEFT | DELETION);
-
-		// right assertions, after changing sides addition becomes deletion
+		assertFileDeletion(c,
+				leftResult.get(0).getChildren().get("folder/a.txt"), "a.txt",
+				LEFT);
+		assertFileDeletion(c,
+				leftResult.get(0).getChildren().get("folder/b.txt"), "b.txt",
+				LEFT);
+		// right asserts, after changing sides addition becomes deletion
 		assertThat(rightResult, notNullValue());
 		assertThat(Integer.valueOf(rightResult.size()), is(Integer.valueOf(1)));
 		assertCommit(rightResult.get(0), c, 2);
-
 		assertThat(rightResult.get(0).getChildren().size(), is(2));
-
-		assertFileAddition(c, rightResult.get(0).getChildren().get("folder/a.txt"), "a.txt", RIGHT | ADDITION);
-		assertFileAddition(c, rightResult.get(0).getChildren().get("folder/b.txt"), "b.txt", RIGHT | ADDITION);
+		assertFileAddition(c,
+				rightResult.get(0).getChildren().get("folder/a.txt"), "a.txt",
+				RIGHT);
+		assertFileAddition(c,
+				rightResult.get(0).getChildren().get("folder/b.txt"), "b.txt",
+				RIGHT);
 	}
 
 	@Test
@@ -249,21 +230,20 @@ public class GitCommitsModelCacheTest extends LocalDiskRepositoryTestCase {
 		RevCommit c1 = commit(git, "first commit");
 		writeTrashFile(db, "a.txt", "new content");
 		RevCommit c2 = commit(git, "second commit");
-
 		// when
-		List<Commit> leftResult = GitCommitsModelCache.build(db, c1, c2);
-		List<Commit> rightResult = GitCommitsModelCache.build(db, c2, c1);
-
+		List<Commit> leftResult = GitCommitsModelCache.build(db, c1, c2, null);
+		List<Commit> rightResult = GitCommitsModelCache.build(db, c2, c1, null);
 		// then
 		// left assertions
 		assertThat(leftResult, notNullValue());
 		assertCommit(leftResult.get(0), c2, 1);
-		assertFileChange(c2, c1, leftResult.get(0).getChildren().get("a.txt"), "a.txt", LEFT | CHANGE);
-
-		// right assertions
+		assertFileChange(c1, c2, leftResult.get(0).getChildren().get("a.txt"),
+				"a.txt", LEFT);
+		// right asserts
 		assertThat(rightResult, notNullValue());
 		assertCommit(rightResult.get(0), c2, 1);
-		assertFileChange(c2, c1, rightResult.get(0).getChildren().get("a.txt"), "a.txt", RIGHT | CHANGE);
+		assertFileChange(c2, c1, rightResult.get(0).getChildren().get("a.txt"),
+				"a.txt", RIGHT);
 	}
 
 	@Test
@@ -275,25 +255,27 @@ public class GitCommitsModelCacheTest extends LocalDiskRepositoryTestCase {
 		RevCommit c1 = commit(git, "first commit");
 		writeTrashFile(db, "folder/a.txt", "new content");
 		RevCommit c2 = commit(git, "second commit");
-
 		// when
-		List<Commit> leftResult = GitCommitsModelCache.build(db, c1, c2);
-		List<Commit> rightResult = GitCommitsModelCache.build(db, c2, c1);
-
+		List<Commit> leftResult = GitCommitsModelCache.build(db, c1, c2, null);
+		List<Commit> rightResult = GitCommitsModelCache.build(db, c2, c1, null);
 		// then
 		// left assertions
 		assertThat(leftResult, notNullValue());
 		assertCommit(leftResult.get(0), c2, 1);
-		assertFileChange(c2, c1, leftResult.get(0).getChildren().get("folder/a.txt"), "a.txt", LEFT | CHANGE);
-
-		// right assertions
+		assertFileChange(c1, c2,
+				leftResult.get(0).getChildren().get("folder/a.txt"), "a.txt",
+				LEFT);
+		// right asserts
 		assertThat(rightResult, notNullValue());
 		assertCommit(rightResult.get(0), c2, 1);
-		assertFileChange(c2, c1, rightResult.get(0).getChildren().get("folder/a.txt"), "a.txt", RIGHT | CHANGE);
+		assertFileChange(c2, c1,
+				rightResult.get(0).getChildren().get("folder/a.txt"), "a.txt",
+				RIGHT);
 	}
 
 	@Test
-	public void shouldListChangesInsideSeparateFoldersInCommit() throws Exception {
+	public void shouldListChangesInsideSeparateFoldersInCommit()
+			throws Exception {
 		// given
 		Git git = new Git(db);
 		writeTrashFile(db, "folder/a.txt", "content");
@@ -304,33 +286,33 @@ public class GitCommitsModelCacheTest extends LocalDiskRepositoryTestCase {
 		writeTrashFile(db, "folder/a.txt", "new content");
 		writeTrashFile(db, "folder2/b.txt", "new b content");
 		RevCommit c2 = commit(git, "second commit");
-
-
 		// when
-		List<Commit> leftResult = GitCommitsModelCache.build(db, c1, c2);
-		List<Commit> rightResult = GitCommitsModelCache.build(db, c2, c1);
-
+		List<Commit> leftResult = GitCommitsModelCache.build(db, c1, c2, null);
+		List<Commit> rightResult = GitCommitsModelCache.build(db, c2, c1, null);
 		// then
 		// left assertions
 		assertThat(leftResult, notNullValue());
 		assertThat(Integer.valueOf(leftResult.size()), is(Integer.valueOf(1)));
 		assertThat(leftResult.get(0).getShortMessage(), is("second commit"));
-
 		assertThat(leftResult.get(0).getChildren(), notNullValue());
 		assertThat(leftResult.get(0).getChildren().size(), is(2));
-
-		assertFileChange(c2, c1, leftResult.get(0).getChildren().get("folder/a.txt"), "a.txt", LEFT | CHANGE);
-		assertFileChange(c2, c1, leftResult.get(0).getChildren().get("folder2/b.txt"), "b.txt",LEFT | CHANGE);
-
-		// right assertions
+		assertFileChange(c1, c2,
+				leftResult.get(0).getChildren().get("folder/a.txt"), "a.txt",
+				LEFT);
+		assertFileChange(c1, c2,
+				leftResult.get(0).getChildren().get("folder2/b.txt"), "b.txt",
+				LEFT);
+		// right asserts
 		assertThat(rightResult, notNullValue());
 		assertThat(Integer.valueOf(rightResult.size()), is(Integer.valueOf(1)));
 		assertThat(rightResult.get(0).getShortMessage(), is("second commit"));
-
 		assertThat(rightResult.get(0).getChildren().size(), is(2));
-
-		assertFileChange(c2, c1, rightResult.get(0).getChildren().get("folder/a.txt"), "a.txt", RIGHT | CHANGE);
-		assertFileChange(c2, c1, rightResult.get(0).getChildren().get("folder2/b.txt"), "b.txt",RIGHT | CHANGE);
+		assertFileChange(c2, c1,
+				rightResult.get(0).getChildren().get("folder/a.txt"), "a.txt",
+				RIGHT);
+		assertFileChange(c2, c1,
+				rightResult.get(0).getChildren().get("folder2/b.txt"), "b.txt",
+				RIGHT);
 	}
 
 	@Test
@@ -344,27 +326,30 @@ public class GitCommitsModelCacheTest extends LocalDiskRepositoryTestCase {
 		writeTrashFile(db, "folder/a.txt", "new content");
 		writeTrashFile(db, "folder/b.txt", "new b content");
 		RevCommit c2 = commit(git, "second commit");
-
 		// when
-		List<Commit> leftResult = GitCommitsModelCache.build(db, c1, c2);
-		List<Commit> rightResult = GitCommitsModelCache.build(db, c2, c1);
-
+		List<Commit> leftResult = GitCommitsModelCache.build(db, c1, c2, null);
+		List<Commit> rightResult = GitCommitsModelCache.build(db, c2, c1, null);
 		// then
 		// left assertions
 		assertThat(leftResult, notNullValue());
 		assertThat(Integer.valueOf(leftResult.size()), is(Integer.valueOf(1)));
 		assertCommit(leftResult.get(0), c2, 2);
-
-		assertFileChange(c2, c1, leftResult.get(0).getChildren().get("folder/a.txt"), "a.txt", LEFT | CHANGE);
-		assertFileChange(c2, c1, leftResult.get(0).getChildren().get("folder/b.txt"), "b.txt", LEFT | CHANGE);
-
-		// right assertions
+		assertFileChange(c1, c2,
+				leftResult.get(0).getChildren().get("folder/a.txt"), "a.txt",
+				LEFT);
+		assertFileChange(c1, c2,
+				leftResult.get(0).getChildren().get("folder/b.txt"), "b.txt",
+				LEFT);
+		// right asserts
 		assertThat(rightResult, notNullValue());
 		assertThat(Integer.valueOf(rightResult.size()), is(Integer.valueOf(1)));
 		assertCommit(rightResult.get(0), c2, 2);
-
-		assertFileChange(c2, c1, rightResult.get(0).getChildren().get("folder/a.txt"), "a.txt", RIGHT | CHANGE);
-		assertFileChange(c2, c1, rightResult.get(0).getChildren().get("folder/b.txt"), "b.txt", RIGHT | CHANGE);
+		assertFileChange(c2, c1,
+				rightResult.get(0).getChildren().get("folder/a.txt"), "a.txt",
+				RIGHT);
+		assertFileChange(c2, c1,
+				rightResult.get(0).getChildren().get("folder/b.txt"), "b.txt",
+				RIGHT);
 	}
 
 	@Test
@@ -381,30 +366,33 @@ public class GitCommitsModelCacheTest extends LocalDiskRepositoryTestCase {
 		writeTrashFile(db, "c.txt", "new c content");
 		git.add().addFilepattern("b.txt").call();
 		RevCommit c2 = commit(git, "second commit");
-
 		// when
-		List<Commit> leftResult = GitCommitsModelCache.build(db, c1, c2);
-		List<Commit> rightResult = GitCommitsModelCache.build(db, c2, c1);
-
+		List<Commit> leftResult = GitCommitsModelCache.build(db, c1, c2, null);
+		List<Commit> rightResult = GitCommitsModelCache.build(db, c2, c1, null);
 		// then
-
-		// left assertions
+		// left asserts
 		assertThat(leftResult, notNullValue());
 		assertCommit(leftResult.get(0), c2, 3);
-		assertFileAddition(c2, c1, leftResult.get(0).getChildren().get("a.txt"), "a.txt", LEFT | ADDITION);
-		assertFileDeletion(c2, c1, leftResult.get(0).getChildren().get("b.txt"), "b.txt", LEFT | DELETION);
-		assertFileChange(c2, c1, leftResult.get(0).getChildren().get("c.txt"), "c.txt", LEFT | CHANGE);
-
-		// right assertions
+		assertFileAddition(c1, c2,
+				leftResult.get(0).getChildren().get("a.txt"), "a.txt", LEFT);
+		assertFileDeletion(c1, c2,
+				leftResult.get(0).getChildren().get("b.txt"), "b.txt", LEFT);
+		assertFileChange(c1, c2, leftResult.get(0).getChildren().get("c.txt"),
+				"c.txt", LEFT);
+		// right asserts
 		assertThat(rightResult, notNullValue());
 		assertCommit(rightResult.get(0), c2, 3);
-		assertFileDeletion(c2, c1, rightResult.get(0).getChildren().get("a.txt"), "a.txt", RIGHT | DELETION);
-		assertFileAddition(c2, c1, rightResult.get(0).getChildren().get("b.txt"), "b.txt", RIGHT | ADDITION);
-		assertFileChange(c2, c1, rightResult.get(0).getChildren().get("c.txt"), "c.txt", RIGHT | CHANGE);
+		assertFileDeletion(c2, c1, rightResult.get(0).getChildren()
+				.get("a.txt"), "a.txt", RIGHT);
+		assertFileAddition(c2, c1, rightResult.get(0).getChildren()
+				.get("b.txt"), "b.txt", RIGHT);
+		assertFileChange(c2, c1, rightResult.get(0).getChildren().get("c.txt"),
+				"c.txt", RIGHT);
 	}
 
 	@Test
-	public void shouldListAllTypeOfChangesInsideFolderInOneCommit() throws Exception {
+	public void shouldListAllTypeOfChangesInsideFolderInOneCommit()
+			throws Exception {
 		// given
 		Git git = new Git(db);
 		writeTrashFile(db, "folder/a.txt", "a content");
@@ -417,29 +405,39 @@ public class GitCommitsModelCacheTest extends LocalDiskRepositoryTestCase {
 		writeTrashFile(db, "folder/c.txt", "new c content");
 		git.add().addFilepattern("folder/b.txt").call();
 		RevCommit c2 = commit(git, "second commit");
-
 		// when
-		List<Commit> leftResult = GitCommitsModelCache.build(db, c1, c2);
-		List<Commit> rightResult = GitCommitsModelCache.build(db, c2, c1);
-
+		List<Commit> leftResult = GitCommitsModelCache.build(db, c1, c2, null);
+		List<Commit> rightResult = GitCommitsModelCache.build(db, c2, c1, null);
 		// then
 		// left assertions
 		assertThat(leftResult, notNullValue());
 		assertCommit(leftResult.get(0), c2, 3);
-		assertFileAddition(c2, c1, leftResult.get(0).getChildren().get("folder/a.txt"), "a.txt", LEFT | ADDITION);
-		assertFileDeletion(c2, c1, leftResult.get(0).getChildren().get("folder/b.txt"), "b.txt", LEFT | DELETION);
-		assertFileChange(c2, c1, leftResult.get(0).getChildren().get("folder/c.txt"), "c.txt", LEFT | CHANGE);
-
-		// right assertions
+		assertFileAddition(c1, c2,
+				leftResult.get(0).getChildren().get("folder/a.txt"), "a.txt",
+				LEFT);
+		assertFileDeletion(c1, c2,
+				leftResult.get(0).getChildren().get("folder/b.txt"), "b.txt",
+				LEFT);
+		assertFileChange(c1, c2,
+				leftResult.get(0).getChildren().get("folder/c.txt"), "c.txt",
+				LEFT);
+		// right asserts
 		assertThat(rightResult, notNullValue());
 		assertCommit(rightResult.get(0), c2, 3);
-		assertFileDeletion(c2, c1, rightResult.get(0).getChildren().get("folder/a.txt"), "a.txt", RIGHT | DELETION);
-		assertFileAddition(c2, c1, rightResult.get(0).getChildren().get("folder/b.txt"), "b.txt", RIGHT | ADDITION);
-		assertFileChange(c2, c1, rightResult.get(0).getChildren().get("folder/c.txt"), "c.txt", RIGHT | CHANGE);
+		assertFileDeletion(c2, c1,
+				rightResult.get(0).getChildren().get("folder/a.txt"), "a.txt",
+				RIGHT);
+		assertFileAddition(c2, c1,
+				rightResult.get(0).getChildren().get("folder/b.txt"), "b.txt",
+				RIGHT);
+		assertFileChange(c2, c1,
+				rightResult.get(0).getChildren().get("folder/c.txt"), "c.txt",
+				RIGHT);
 	}
 
 	@Test
-	public void shouldListAllTypeOfChangesInsideSeparateFoldersInOneCommit() throws Exception {
+	public void shouldListAllTypeOfChangesInsideSeparateFoldersInOneCommit()
+			throws Exception {
 		// given
 		Git git = new Git(db);
 		writeTrashFile(db, "folder/a.txt", "a content");
@@ -452,73 +450,80 @@ public class GitCommitsModelCacheTest extends LocalDiskRepositoryTestCase {
 		writeTrashFile(db, "folder2/c.txt", "new c content");
 		git.add().addFilepattern("folder1/b.txt").call();
 		RevCommit c2 = commit(git, "second commit");
-
-
 		// when
-		List<Commit> leftResult = GitCommitsModelCache.build(db, c1, c2);
-		List<Commit> rightResult = GitCommitsModelCache.build(db, c2, c1);
-
+		List<Commit> leftResult = GitCommitsModelCache.build(db, c1, c2, null);
+		List<Commit> rightResult = GitCommitsModelCache.build(db, c2, c1, null);
 		// then
 		// left assertions
 		assertThat(leftResult, notNullValue());
 		assertThat(Integer.valueOf(leftResult.size()), is(Integer.valueOf(1)));
 		assertThat(leftResult.get(0).getShortMessage(), is("second commit"));
-
 		assertThat(leftResult.get(0).getChildren(), notNullValue());
 		assertThat(leftResult.get(0).getChildren().size(), is(3));
-
-		assertFileAddition(c2, c1, leftResult.get(0).getChildren().get("folder/a.txt"), "a.txt", LEFT | ADDITION);
-		assertFileDeletion(c2, c1, leftResult.get(0).getChildren().get("folder1/b.txt"), "b.txt", LEFT | DELETION);
-		assertFileChange(c2, c1, leftResult.get(0).getChildren().get("folder2/c.txt"), "c.txt", LEFT | CHANGE);
-
-		// right assertions
+		assertFileAddition(c1, c2,
+				leftResult.get(0).getChildren().get("folder/a.txt"), "a.txt",
+				LEFT);
+		assertFileDeletion(c1, c2,
+				leftResult.get(0).getChildren().get("folder1/b.txt"), "b.txt",
+				LEFT);
+		assertFileChange(c1, c2,
+				leftResult.get(0).getChildren().get("folder2/c.txt"), "c.txt",
+				LEFT);
+		// right asserts
 		assertThat(rightResult, notNullValue());
 		assertThat(Integer.valueOf(rightResult.size()), is(Integer.valueOf(1)));
 		assertThat(rightResult.get(0).getShortMessage(), is("second commit"));
-
 		assertThat(rightResult.get(0).getChildren(), notNullValue());
 		assertThat(rightResult.get(0).getChildren().size(), is(3));
-
-		assertFileDeletion(c2, c1, rightResult.get(0).getChildren().get("folder/a.txt"), "a.txt", RIGHT | DELETION);
-		assertFileAddition(c2, c1, rightResult.get(0).getChildren().get("folder1/b.txt"), "b.txt",RIGHT | ADDITION);
-		assertFileChange(c2, c1, rightResult.get(0).getChildren().get("folder2/c.txt"), "c.txt", RIGHT | CHANGE);
+		assertFileDeletion(c2, c1,
+				rightResult.get(0).getChildren().get("folder/a.txt"), "a.txt",
+				RIGHT);
+		assertFileAddition(c2, c1,
+				rightResult.get(0).getChildren().get("folder1/b.txt"), "b.txt",
+				RIGHT);
+		assertFileChange(c2, c1,
+				rightResult.get(0).getChildren().get("folder2/c.txt"), "c.txt",
+				RIGHT);
 	}
 
 	private RevCommit commit(Git git, String msg) throws Exception {
 		tick();
-		return git.commit().setAll(true).setMessage(msg).setCommitter(committer).call();
+		return git.commit().setAll(true).setMessage(msg)
+				.setCommitter(committer).call();
 	}
 
-	private ObjectId initialTagId() throws AmbiguousObjectException, IOException {
+	private ObjectId initialTagId() throws AmbiguousObjectException,
+			IOException {
 		return db.resolve(INITIAL_TAG);
 	}
 
-	private void assertEmptyCommit(Commit commit, RevCommit actualCommit, int direction) {
-		commonCommitAsserts(commit, actualCommit);
-		assertThat(commit.getChildren(), nullValue());
-		assertThat(commit.getDirection(), is(direction));
-	}
-
-	private void assertCommit(Commit commit, RevCommit actualCommit, int childrenCount) {
+	private void assertCommit(Commit commit, RevCommit actualCommit,
+			int childrenCount) {
 		commonCommitAsserts(commit, actualCommit);
 		assertThat(commit.getChildren(), notNullValue());
 		assertThat(commit.getChildren().size(), is(childrenCount));
-
 	}
 
 	private void commonCommitAsserts(Commit commit, RevCommit actualCommit) {
 		assertThat(commit.getShortMessage(), is(actualCommit.getShortMessage()));
 		assertThat(commit.getId().toObjectId(), is(actualCommit.getId()));
-		assertThat(commit.getAuthorName(), is(actualCommit.getAuthorIdent().getName()));
-		assertThat(commit.getCommitterName(), is(actualCommit.getCommitterIdent().getName()));
-		assertThat(commit.getCommitDate(), is(actualCommit.getAuthorIdent().getWhen()));
+		assertThat(commit.getAuthorName(), is(actualCommit.getAuthorIdent()
+				.getName()));
+		assertThat(commit.getCommitterName(), is(actualCommit
+				.getCommitterIdent().getName()));
+		assertThat(commit.getCommitDate(), is(actualCommit.getAuthorIdent()
+				.getWhen()));
 	}
 
 	private void assertFileChange(RevCommit actual, RevCommit parent,
 			Change change, String name, int direction) {
-		commonFileAssertions(actual, parent, change, name, direction);
+		commonFileAssertions(actual, parent, change, name);
 		assertThat(change.getObjectId(), not(ZERO_ID));
 		assertThat(change.getRemoteObjectId(), not(ZERO_ID));
+		if (direction == LEFT)
+			assertThat(change.getKind(), is(LEFT | CHANGE));
+		else
+			assertThat(change.getKind(), is(RIGHT | CHANGE));
 	}
 
 	private void assertFileAddition(RevCommit actual, Change change,
@@ -528,33 +533,45 @@ public class GitCommitsModelCacheTest extends LocalDiskRepositoryTestCase {
 
 	private void assertFileAddition(RevCommit actual, RevCommit parent,
 			Change change, String name, int direction) {
-		commonFileAssertions(actual, parent, change, name, direction);
-		assertThat(change.getObjectId(), not(ZERO_ID));
-		assertThat(change.getRemoteObjectId(), nullValue());
+		commonFileAssertions(actual, parent, change, name);
+		if (direction == LEFT) {
+			assertThat(change.getKind(), is(LEFT | ADDITION));
+			assertThat(change.getRemoteCommitId(), not(ZERO_ID));
+			assertThat(change.getObjectId(), nullValue());
+		} else { // should be Differencer.Right
+			assertThat(change.getKind(), is(RIGHT | ADDITION));
+			assertThat(change.getObjectId(), not(ZERO_ID));
+			assertThat(change.getRemoteObjectId(), nullValue());
+		}
 	}
 
-	private void assertFileDeletion(RevCommit actual, Change change,
+	private void assertFileDeletion(RevCommit parent, Change change,
 			String name, int direction) {
-		assertFileDeletion(actual, null, change, name, direction);
-
+		assertFileDeletion(null, parent, change, name, direction);
 	}
 
 	private void assertFileDeletion(RevCommit actual, RevCommit parent,
 			Change change, String name, int direction) {
-		commonFileAssertions(actual, parent, change, name, direction);
-		assertThat(change.getObjectId(), nullValue());
-		assertThat(change.getRemoteObjectId(), not(ZERO_ID));
+		commonFileAssertions(actual, parent, change, name);
+		if (direction == LEFT) {
+			assertThat(change.getRemoteObjectId(), nullValue());
+			assertThat(change.getObjectId(), not(ZERO_ID));
+			assertThat(change.getKind(), is(LEFT | DELETION));
+		} else { // should be Differencer.Right
+			assertThat(change.getKind(), is(RIGHT | DELETION));
+			assertThat(change.getObjectId(), nullValue());
+			assertThat(change.getRemoteObjectId(), not(ZERO_ID));
+		}
 	}
 
 	private void commonFileAssertions(RevCommit actual, RevCommit parent,
-			Change change, String name, int direction) {
+			Change change, String name) {
 		assertThat(change, notNullValue());
-		assertThat(change.getCommitId().toObjectId(), is(actual.getId()));
 		if (parent != null)
 			assertThat(change.getRemoteCommitId().toObjectId(),
 					is(parent.getId()));
+		if (actual != null && !ObjectId.zeroId().equals(actual))
+			assertThat(change.getCommitId().toObjectId(), is(actual.getId()));
 		assertThat(change.getName(), is(name));
-		assertThat(change.getKind(), is(direction));
 	}
-
 }
