@@ -12,7 +12,6 @@ import static org.junit.Assert.assertEquals;
 
 import java.io.ByteArrayInputStream;
 import java.io.File;
-import java.io.IOException;
 
 import org.eclipse.core.filesystem.EFS;
 import org.eclipse.core.resources.IFile;
@@ -62,7 +61,7 @@ public class DiscardChangesOperationTest extends DualRepositoryTestCase {
 						repository1
 								.track(EFS.getStore(resource.getLocationURI())
 										.toLocalFile(0, null));
-					} catch (IOException e) {
+					} catch (Exception e) {
 						throw new CoreException(Activator.error(e.getMessage(),
 								e));
 					}
