@@ -19,4 +19,13 @@ public class JobFamilies {
 	 */
 	public static final Object REPOSITORY_CHANGED = new Object();
 
+	/**
+	 * Job family for Index Diff Cache update
+	 */
+	public static final Object INDEX_DIFF_CACHE_UPDATE = new Object();
+
+	/**
+	 * Job family for auto share job
+	 */
+	public static final Object AUTO_SHARE = new Object();
 }
