@@ -31,6 +31,7 @@ import org.eclipse.core.resources.ResourcesPlugin;
 import org.eclipse.core.runtime.Path;
 import org.eclipse.egit.core.Activator;
 import org.eclipse.egit.core.RepositoryCache;
+import org.eclipse.egit.core.op.AddToIndexOperation;
 import org.eclipse.egit.core.op.CloneOperation;
 import org.eclipse.egit.core.op.CommitOperation;
 import org.eclipse.egit.core.op.ConnectProviderOperation;
@@ -82,7 +83,7 @@ import org.junit.BeforeClass;
  * <p>
  * A typical code sequence for setting up these two repositories could look
  * like:
- * 
+ *
  * <pre>
  *  private File localRepo;
  *  private File remoteRepo;
@@ -124,7 +125,7 @@ public abstract class LocalRepositoryTestCase extends EGitTestCase {
 	public static File getTestDirectory() {
 		return testDirectory;
 	}
-	
+
 	@BeforeClass
 	public static void beforeClassBase() throws Exception {
 		deleteAllProjects();
@@ -223,19 +224,19 @@ public abstract class LocalRepositoryTestCase extends EGitTestCase {
 
 		new ConnectProviderOperation(firstProject, gitDir).execute(null);
 
-		IProject secondPoject = ResourcesPlugin.getWorkspace().getRoot()
+		IProject secondProject = ResourcesPlugin.getWorkspace().getRoot()
 				.getProject(PROJ2);
 
-		if (secondPoject.exists())
-			secondPoject.delete(true, null);
+		if (secondProject.exists())
+			secondProject.delete(true, null);
 
 		desc = ResourcesPlugin.getWorkspace().newProjectDescription(PROJ2);
 		desc.setLocation(new Path(new File(myRepository.getWorkTree(), PROJ2)
 				.getPath()));
-		secondPoject.create(desc, null);
-		secondPoject.open(null);
+		secondProject.create(desc, null);
+		secondProject.open(null);
 
-		IFolder secondfolder = secondPoject.getFolder(FOLDER);
+		IFolder secondfolder = secondProject.getFolder(FOLDER);
 		secondfolder.create(false, true, null);
 		IFile secondtextFile = secondfolder.getFile(FILE1);
 		secondtextFile.create(new ByteArrayInputStream("Hello, world"
@@ -248,7 +249,7 @@ public abstract class LocalRepositoryTestCase extends EGitTestCase {
 		// gitignore.create(new ByteArrayInputStream("/.project\n"
 		// .getBytes(firstProject.getDefaultCharset())), false, null);
 
-		new ConnectProviderOperation(secondPoject, gitDir).execute(null);
+		new ConnectProviderOperation(secondProject, gitDir).execute(null);
 
 		IFile[] commitables = new IFile[] { firstProject.getFile(".project"),
 				textFile, textFile2, secondtextFile, secondtextFile2 };
@@ -380,8 +381,8 @@ public abstract class LocalRepositoryTestCase extends EGitTestCase {
 				return name.equals(".project");
 			}
 		};
-		for (File file : myRepository.getWorkTree().listFiles()) {
-			if (file.isDirectory()) {
+		for (File file : myRepository.getWorkTree().listFiles())
+			if (file.isDirectory())
 				if (file.list(projectFilter).length > 0) {
 					IProjectDescription desc = ResourcesPlugin.getWorkspace()
 							.newProjectDescription(file.getName());
@@ -394,8 +395,6 @@ public abstract class LocalRepositoryTestCase extends EGitTestCase {
 					new ConnectProviderOperation(prj, myRepository
 							.getDirectory()).execute(null);
 				}
-			}
-		}
 	}
 
 	@SuppressWarnings("boxing")
@@ -413,19 +412,29 @@ public abstract class LocalRepositoryTestCase extends EGitTestCase {
 	}
 
 	/**
+	 * Modify with a random content and commit.
+	 *
 	 * @param commitMessage
 	 *            may be null
 	 * @throws Exception
 	 */
 	protected static void touchAndSubmit(String commitMessage) throws Exception {
-		IProject prj = ResourcesPlugin.getWorkspace().getRoot().getProject(
-				PROJ1);
-		if (!prj.isAccessible())
-			throw new IllegalStateException("No project to touch");
-		IFile file = prj.getFile(new Path("folder/test.txt"));
 		String newContent = "Touched at " + System.currentTimeMillis();
-		file.setContents(new ByteArrayInputStream(newContent.getBytes(prj
-				.getDefaultCharset())), 0, null);
+		touchAndSubmit(newContent, commitMessage);
+	}
+
+	/**
+	 * Modify with the given content and commit.
+	 *
+	 * @param newContent
+	 *            new file content
+	 * @param commitMessage
+	 *            may be null
+	 * @throws Exception
+	 */
+	protected static void touchAndSubmit(String newContent, String commitMessage)
+			throws Exception {
+		IFile file = touch(newContent);
 
 		IFile[] commitables = new IFile[] { file };
 		ArrayList<IFile> untracked = new ArrayList<IFile>();
@@ -441,6 +450,50 @@ public abstract class LocalRepositoryTestCase extends EGitTestCase {
 		op.execute(null);
 	}
 
+	/**
+	 * Modify with the given content.
+	 *
+	 * @param newContent
+	 *            new file content
+	 * @return the modified file
+	 * @throws Exception
+	 */
+	protected static IFile touch(final String newContent) throws Exception {
+		return touch(PROJ1, "folder/test.txt", newContent);
+	}
+
+	/**
+	 * Modify the specified file with the given content.
+	 *
+	 * @param projectName
+	 *            project name
+	 * @param filePath
+	 *            file path under the given project
+	 * @param newContent
+	 *            new file content
+	 * @return the modified file
+	 * @throws Exception
+	 */
+	protected static IFile touch(String projectName, String filePath,
+			String newContent) throws Exception {
+		IProject prj = ResourcesPlugin.getWorkspace().getRoot()
+				.getProject(projectName);
+		if (!prj.isAccessible())
+			throw new IllegalStateException("No project to touch");
+		IFile file = prj.getFile(new Path(filePath));
+		file.setContents(
+				new ByteArrayInputStream(newContent.getBytes(prj
+						.getDefaultCharset())), 0, null);
+		return file;
+	}
+
+	protected static void stage(IFile file) throws Exception {
+		ArrayList<IFile> unstaged = new ArrayList<IFile>();
+		unstaged.addAll(Arrays.asList(new IFile[] { file }));
+		AddToIndexOperation op = new AddToIndexOperation(unstaged);
+		op.execute(null);
+	}
+
 	protected static void addAndCommit(IFile file, String commitMessage)
 			throws Exception {
 		IProject prj = file.getProject();
@@ -472,9 +525,8 @@ public abstract class LocalRepositoryTestCase extends EGitTestCase {
 		if (file.exists()) {
 			byte[] bytes = IO.readFully(file.getLocation().toFile());
 			return new String(bytes, file.getCharset());
-		} else {
+		} else
 			return "";
-		}
 	}
 
 	/**
@@ -485,7 +537,7 @@ public abstract class LocalRepositoryTestCase extends EGitTestCase {
 	 */
 	protected SWTBotTreeItem getProjectItem(SWTBotTree projectExplorerTree,
 			String project) {
-		return new TestUtil().getProjectItem(projectExplorerTree, project);
+		return new TestUtil().getProjectItems(projectExplorerTree, project)[0];
 	}
 
 	protected void pressAltAndChar(SWTBotShell shell, char charToPress) {
