@@ -46,6 +46,7 @@ import org.eclipse.egit.ui.internal.repository.tree.TagsNode;
 import org.eclipse.egit.ui.test.ContextMenuHelper;
 import org.eclipse.egit.ui.test.TestUtil;
 import org.eclipse.egit.ui.view.repositories.GitRepositoriesViewTestUtils;
+import org.eclipse.jface.bindings.keys.KeyStroke;
 import org.eclipse.jface.dialogs.IDialogConstants;
 import org.eclipse.jgit.lib.Constants;
 import org.eclipse.jgit.lib.ObjectId;
@@ -61,6 +62,7 @@ import org.eclipse.swtbot.swt.finder.junit.SWTBotJunit4ClassRunner;
 import org.eclipse.swtbot.swt.finder.utils.TableCollection;
 import org.eclipse.swtbot.swt.finder.waits.ICondition;
 import org.eclipse.swtbot.swt.finder.widgets.SWTBotShell;
+import org.eclipse.swtbot.swt.finder.widgets.SWTBotToolbarDropDownButton;
 import org.eclipse.swtbot.swt.finder.widgets.SWTBotTree;
 import org.eclipse.swtbot.swt.finder.widgets.SWTBotTreeItem;
 import org.eclipse.swtbot.swt.finder.widgets.TimeoutException;
@@ -212,8 +214,10 @@ public class BranchAndResetActionTest extends LocalRepositoryTestCase {
 				}
 			});
 
-			showUndeleted.bot().radio(
-					UIText.NonDeletedFilesTree_FileSystemPathsButton).click();
+			SWTBotToolbarDropDownButton pathButton = showUndeleted.bot().toolbarDropDownButton();
+			pathButton.menuItem(UIText.NonDeletedFilesTree_FileSystemPathsButton).click();
+			// see http://www.eclipse.org/forums/index.php/t/159133/ why we need this
+			pathButton.pressShortcut(KeyStroke.getInstance("ESC"));
 			// fs path
 			IPath path = new Path(lookupRepository(repositoryFile)
 					.getWorkTree().getPath()).append(PROJ1).append(FOLDER)
@@ -222,18 +226,17 @@ public class BranchAndResetActionTest extends LocalRepositoryTestCase {
 			for (int i = 0; i < path.segmentCount(); i++) {
 				boolean found = false;
 				String segment = path.segment(i);
-				for (SWTBotTreeItem item : items) {
+				for (SWTBotTreeItem item : items)
 					if (item.getText().equals(segment)) {
 						found = true;
 						items = item.getItems();
 					}
-				}
 				assertTrue(found);
 			}
-
+			pathButton.menuItem(UIText.NonDeletedFilesTree_ResourcePathsButton).click();
+			// see http://www.eclipse.org/forums/index.php/t/159133/ why we need this
+			pathButton.pressShortcut(KeyStroke.getInstance("ESC"));
 			// resource path
-			showUndeleted.bot().radio(
-					UIText.NonDeletedFilesTree_ResourcePathsButton).click();
 			assertEquals("ToBeDeleted", showUndeleted.bot().tree()
 					.getAllItems()[0].getItems()[0].getItems()[0].getText());
 			Display.getDefault().syncExec(new Runnable() {
