@@ -18,6 +18,7 @@ import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Iterator;
 import java.util.LinkedHashSet;
+import java.util.List;
 import java.util.Set;
 
 import org.eclipse.core.commands.AbstractHandler;
@@ -31,6 +32,7 @@ import org.eclipse.core.resources.mapping.ResourceMapping;
 import org.eclipse.core.resources.mapping.ResourceTraversal;
 import org.eclipse.core.runtime.CoreException;
 import org.eclipse.core.runtime.IAdaptable;
+import org.eclipse.core.runtime.PlatformObject;
 import org.eclipse.egit.core.project.RepositoryMapping;
 import org.eclipse.egit.ui.Activator;
 import org.eclipse.egit.ui.UIText;
@@ -39,10 +41,19 @@ import org.eclipse.jface.text.TextSelection;
 import org.eclipse.jface.viewers.ISelection;
 import org.eclipse.jface.viewers.IStructuredSelection;
 import org.eclipse.jface.viewers.StructuredSelection;
+import org.eclipse.jgit.diff.DiffEntry;
+import org.eclipse.jgit.diff.DiffEntry.ChangeType;
+import org.eclipse.jgit.diff.RenameDetector;
 import org.eclipse.jgit.lib.Constants;
+import org.eclipse.jgit.lib.NullProgressMonitor;
+import org.eclipse.jgit.lib.ObjectReader;
 import org.eclipse.jgit.lib.Ref;
 import org.eclipse.jgit.lib.Repository;
 import org.eclipse.jgit.lib.RepositoryState;
+import org.eclipse.jgit.revwalk.FollowFilter;
+import org.eclipse.jgit.revwalk.RevCommit;
+import org.eclipse.jgit.revwalk.RevWalk;
+import org.eclipse.jgit.treewalk.TreeWalk;
 import org.eclipse.osgi.util.NLS;
 import org.eclipse.swt.widgets.Shell;
 import org.eclipse.ui.IEditorInput;
@@ -90,12 +101,14 @@ abstract class RepositoryActionHandler extends AbstractHandler {
 		return ret.toArray(new IProject[ret.size()]);
 	}
 
-	private Set<IProject> extractProjectsFromMappings(IStructuredSelection selection) {
+	private Set<IProject> extractProjectsFromMappings(
+			IStructuredSelection selection) {
 		Set<IProject> ret = new LinkedHashSet<IProject>();
 		for (ResourceMapping mapping : (ResourceMapping[]) getSelectedAdaptables(
 				selection, ResourceMapping.class)) {
 			IProject[] projects = mapping.getProjects();
-			ret.addAll(Arrays.asList(projects));
+			if (projects != null)
+				ret.addAll(Arrays.asList(projects));
 		}
 		return ret;
 	}
@@ -175,12 +188,11 @@ abstract class RepositoryActionHandler extends AbstractHandler {
 				.getProjects();
 		for (IProject project : projects) {
 			RepositoryMapping mapping = RepositoryMapping.getMapping(project);
-			for (Repository repository : repositories) {
+			for (Repository repository : repositories)
 				if (mapping != null && mapping.getRepository() == repository) {
 					ret.add(project);
 					break;
 				}
-			}
 		}
 		return ret.toArray(new IProject[ret.size()]);
 	}
@@ -244,7 +256,30 @@ abstract class RepositoryActionHandler extends AbstractHandler {
 				return null;
 			}
 		}
-		if (mapping == null) {
+		Repository result = null;
+		if (mapping == null)
+			for (Object o : selection.toArray()) {
+				Repository nextRepo = null;
+				if (o instanceof Repository)
+					nextRepo = (Repository) o;
+				else if (o instanceof PlatformObject)
+					nextRepo = (Repository) ((PlatformObject) o)
+							.getAdapter(Repository.class);
+				if (nextRepo != null && result != null
+						&& !result.equals(nextRepo)) {
+					if (warn)
+						MessageDialog
+								.openError(
+										shell,
+										UIText.RepositoryAction_multiRepoSelectionTitle,
+										UIText.RepositoryAction_multiRepoSelection);
+					return null;
+				}
+				result = nextRepo;
+			}
+		else
+			result = mapping.getRepository();
+		if (result == null) {
 			if (warn)
 				MessageDialog.openError(shell,
 						UIText.RepositoryAction_errorFindingRepoTitle,
@@ -252,8 +287,7 @@ abstract class RepositoryActionHandler extends AbstractHandler {
 			return null;
 		}
 
-		final Repository repository = mapping.getRepository();
-		return repository;
+		return result;
 	}
 
 	/**
@@ -267,8 +301,24 @@ abstract class RepositoryActionHandler extends AbstractHandler {
 	 */
 	protected Repository[] getRepositories(ExecutionEvent event)
 			throws ExecutionException {
-		IProject[] selectedProjects = getSelectedProjects(event);
-		return getRepositoriesFor(selectedProjects);
+		IProject[] selectedProjects = getProjectsForSelectedResources(event);
+		if (selectedProjects.length > 0)
+			return getRepositoriesFor(selectedProjects);
+		IStructuredSelection selection = getSelection(event);
+		if (!selection.isEmpty()) {
+			Set<Repository> repos = new LinkedHashSet<Repository>();
+			for (Object o : selection.toArray())
+				if (o instanceof Repository)
+					repos.add((Repository) o);
+				else if (o instanceof PlatformObject) {
+					Repository repo = (Repository) ((PlatformObject) o)
+							.getAdapter(Repository.class);
+					if (repo != null)
+						repos.add(repo);
+				}
+			return repos.toArray(new Repository[repos.size()]);
+		}
+		return new Repository[0];
 	}
 
 	/**
@@ -278,8 +328,24 @@ abstract class RepositoryActionHandler extends AbstractHandler {
 	 * @return repositories for selection, or an empty array
 	 */
 	protected Repository[] getRepositories() {
-		IProject[] selectedProjects = getSelectedProjects(getSelection());
-		return getRepositoriesFor(selectedProjects);
+		IProject[] selectedProjects = getProjectsForSelectedResources();
+		if (selectedProjects.length > 0)
+			return getRepositoriesFor(selectedProjects);
+		IStructuredSelection selection = getSelection();
+		if (!selection.isEmpty()) {
+			Set<Repository> repos = new LinkedHashSet<Repository>();
+			for (Object o : selection.toArray())
+				if (o instanceof Repository)
+					repos.add((Repository) o);
+				else if (o instanceof PlatformObject) {
+					Repository repo = (Repository) ((PlatformObject) o)
+							.getAdapter(Repository.class);
+					if (repo != null)
+						repos.add(repo);
+				}
+			return repos.toArray(new Repository[repos.size()]);
+		}
+		return new Repository[0];
 	}
 
 	/**
@@ -297,8 +363,8 @@ abstract class RepositoryActionHandler extends AbstractHandler {
 		if (selection == null)
 			selection = HandlerUtil.getCurrentSelectionChecked(event);
 		if (selection instanceof TextSelection) {
-			IEditorInput editorInput = (IEditorInput) HandlerUtil
-					.getVariable(event, ISources.ACTIVE_EDITOR_INPUT_NAME);
+			IEditorInput editorInput = (IEditorInput) HandlerUtil.getVariable(
+					event, ISources.ACTIVE_EDITOR_INPUT_NAME);
 			IResource resource = ResourceUtil.getResource(editorInput);
 			if (resource != null)
 				return new StructuredSelection(resource);
@@ -332,7 +398,7 @@ abstract class RepositoryActionHandler extends AbstractHandler {
 		Object selection;
 		if (aSelection == null && ctx != null) {
 			selection = ctx.getVariable(ISources.ACTIVE_MENU_SELECTION_NAME);
-			if (selection == null)
+			if (!(selection instanceof ISelection))
 				selection = ctx
 						.getVariable(ISources.ACTIVE_CURRENT_SELECTION_NAME);
 		} else if (aSelection != null)
@@ -385,52 +451,28 @@ abstract class RepositoryActionHandler extends AbstractHandler {
 			Iterator elements = ((IStructuredSelection) selection).iterator();
 			while (elements.hasNext()) {
 				Object adapter = getAdapter(elements.next(), c);
-				if (c.isInstance(adapter)) {
+				if (c.isInstance(adapter))
 					result.add(adapter);
-				}
 			}
 		}
-		if (result != null && !result.isEmpty()) {
-			return result.toArray((Object[]) Array
-					.newInstance(c, result.size()));
-		}
+		if (result != null && !result.isEmpty())
+			return result
+					.toArray((Object[]) Array.newInstance(c, result.size()));
 		return (Object[]) Array.newInstance(c, 0);
 	}
 
 	private Object getAdapter(Object adaptable, Class c) {
-		if (c.isInstance(adaptable)) {
+		if (c.isInstance(adaptable))
 			return adaptable;
-		}
 		if (adaptable instanceof IAdaptable) {
 			IAdaptable a = (IAdaptable) adaptable;
 			Object adapter = a.getAdapter(c);
-			if (c.isInstance(adapter)) {
+			if (c.isInstance(adapter))
 				return adapter;
-			}
 		}
 		return null;
 	}
 
-	private IProject[] getSelectedProjects(ExecutionEvent event)
-			throws ExecutionException {
-		IStructuredSelection selection = getSelection(event);
-		return getSelectedProjects(selection);
-	}
-
-	private IProject[] getSelectedProjects(IStructuredSelection selection) {
-		IResource[] selectedResources = getSelectedResources(selection);
-		if (selectedResources.length == 0)
-			return new IProject[0];
-		ArrayList<IProject> projects = new ArrayList<IProject>();
-		for (int i = 0; i < selectedResources.length; i++) {
-			IResource resource = selectedResources[i];
-			if (resource.getType() == IResource.PROJECT) {
-				projects.add((IProject) resource);
-			}
-		}
-		return projects.toArray(new IProject[projects.size()]);
-	}
-
 	/**
 	 * @param event
 	 * @return the resources in the selection
@@ -467,7 +509,8 @@ abstract class RepositoryActionHandler extends AbstractHandler {
 	}
 
 	private void extractResourcesFromMapping(Set<IResource> result, Object o) {
-		ResourceMapping mapping = (ResourceMapping) getAdapter(o, ResourceMapping.class);
+		ResourceMapping mapping = (ResourceMapping) getAdapter(o,
+				ResourceMapping.class);
 		if (mapping != null) {
 			ResourceTraversal[] traversals;
 			try {
@@ -539,25 +582,116 @@ abstract class RepositoryActionHandler extends AbstractHandler {
 			message = e.getMessage();
 		}
 
-		if (message != null) {
+		if (message != null)
 			MessageDialog.openError(getShell(event),
 					UIText.MergeAction_CannotMerge, message);
-		}
 		return (message == null);
 	}
 
 	/**
 	 *
+	 * @param repository
+	 *            the repository to check
 	 * @return {@code true} when {@link Constants#HEAD} can be resolved,
 	 *         {@code false} otherwise
 	 */
-	protected boolean containsHead() {
+	protected boolean containsHead(Repository repository) {
 		try {
-			return getRepository().resolve(Constants.HEAD) != null;
+			return repository != null ? repository.resolve(Constants.HEAD) != null
+					: false;
 		} catch (Exception e) {
 			// do nothing
 		}
 
 		return false;
 	}
+
+	protected boolean isLocalBranchCheckedout(Repository repository) {
+		try {
+			return repository.getFullBranch().startsWith(Constants.R_HEADS);
+		} catch (Exception e) {
+			// do nothing
+		}
+
+		return false;
+	}
+
+	protected String getPreviousPath(Repository repository,
+			ObjectReader reader, RevCommit headCommit,
+			RevCommit previousCommit, String path) throws IOException {
+		TreeWalk walk = new TreeWalk(reader);
+		walk.setRecursive(true);
+		walk.addTree(previousCommit.getTree());
+		walk.addTree(headCommit.getTree());
+
+		List<DiffEntry> entries = DiffEntry.scan(walk);
+		if (entries.size() < 2)
+			return path;
+
+		for (DiffEntry diff : entries)
+			if (diff.getChangeType() == ChangeType.MODIFY
+					&& path.equals(diff.getNewPath()))
+				return path;
+
+		RenameDetector detector = new RenameDetector(repository);
+		detector.addAll(entries);
+		List<DiffEntry> renames = detector.compute(walk.getObjectReader(),
+				NullProgressMonitor.INSTANCE);
+		for (DiffEntry diff : renames)
+			if (diff.getChangeType() == ChangeType.RENAME
+					&& path.equals(diff.getNewPath()))
+				return diff.getOldPath();
+
+		return path;
+	}
+
+	protected List<PreviousCommit> findPreviousCommits() throws IOException {
+		List<PreviousCommit> result = new ArrayList<PreviousCommit>();
+		Repository repository = getRepository();
+		IResource resource = getSelectedResources()[0];
+		String path = RepositoryMapping.getMapping(resource.getProject())
+				.getRepoRelativePath(resource);
+		RevWalk rw = new RevWalk(repository);
+		try {
+			if (path.length() > 0) {
+				FollowFilter filter = FollowFilter.create(path);
+				rw.setTreeFilter(filter);
+			}
+
+			RevCommit headCommit = rw.parseCommit(repository.getRef(
+					Constants.HEAD).getObjectId());
+			rw.markStart(headCommit);
+			headCommit = rw.next();
+
+			if (headCommit == null)
+				return result;
+			List<RevCommit> directParents = Arrays.asList(headCommit
+					.getParents());
+
+			RevCommit previousCommit = rw.next();
+			while (previousCommit != null && result.size() < directParents.size()) {
+				if (directParents.contains(previousCommit)) {
+					String previousPath = getPreviousPath(repository,
+							rw.getObjectReader(), headCommit, previousCommit,
+							path);
+					result.add(new PreviousCommit(previousCommit, previousPath));
+				}
+				previousCommit = rw.next();
+			}
+		} finally {
+			rw.dispose();
+		}
+		return result;
+	}
+
+	// keep track of the path of an ancestor (for following renames)
+	protected static final class PreviousCommit {
+		final RevCommit commit;
+		final String path;
+		PreviousCommit(final RevCommit commit, final String path) {
+			this.commit = commit;
+			this.path = path;
+		}
+	}
+
 }
