@@ -35,6 +35,7 @@ import org.eclipse.egit.ui.Activator;
 import org.eclipse.egit.ui.UIIcons;
 import org.eclipse.egit.ui.UIText;
 import org.eclipse.egit.ui.internal.clone.GitCloneSourceProviderExtension.CloneSourceProvider;
+import org.eclipse.egit.ui.internal.provisional.wizards.GitRepositoryInfo;
 import org.eclipse.egit.ui.internal.provisional.wizards.IRepositorySearchResult;
 import org.eclipse.egit.ui.internal.provisional.wizards.NoRepositoryInfoException;
 import org.eclipse.jface.operation.IRunnableWithProgress;
@@ -57,14 +58,15 @@ public class GitImportWizard extends AbstractGitCloneWizard implements IImportWi
 
 	private GitSelectWizardPage importWithDirectoriesPage = new GitSelectWizardPage(){
 		public void setVisible(boolean visible) {
-			if (visible && (cloneDestination.cloneSettingsChanged())) {
+			if (existingRepo == null && visible && (cloneDestination.cloneSettingsChanged())) {
 				setCallerRunsCloneOperation(true);
 				try {
-					performClone(currentSearchResult.getGitRepositoryInfo());
+					final GitRepositoryInfo repositoryInfo = currentSearchResult.getGitRepositoryInfo();
+					performClone(repositoryInfo);
 					importWithDirectoriesPage.getControl().getDisplay().asyncExec(new Runnable() {
 
 						public void run() {
-							runCloneOperation(getContainer());
+							runCloneOperation(getContainer(), repositoryInfo);
 							cloneDestination.saveSettingsForClonedRepo();
 						}});
 				} catch (URISyntaxException e) {
@@ -83,6 +85,8 @@ public class GitImportWizard extends AbstractGitCloneWizard implements IImportWi
 
 	private GitCreateGeneralProjectPage createGeneralProjectPage = new GitCreateGeneralProjectPage();
 
+	private Repository existingRepo;
+
 	/**
 	 * The default constructor
 	 */
@@ -129,14 +133,16 @@ public class GitImportWizard extends AbstractGitCloneWizard implements IImportWi
 
 	@Override
 	public IWizardPage getNextPage(IWizardPage page) {
-		if (page == selectRepoPage ) {
+		if (page == selectRepoPage) {
+			existingRepo = selectRepoPage.getRepository();
 			importWithDirectoriesPage.setRepository(selectRepoPage
 					.getRepository());
 			return importWithDirectoriesPage;
 		} else if (page == cloneDestination) {
-			importWithDirectoriesPage.setRepository(getClonedRepository());
+			existingRepo = null;
+			importWithDirectoriesPage.setRepository(getTargetRepository());
 			return importWithDirectoriesPage;
-		} else if (page == importWithDirectoriesPage) {
+		} else if (page == importWithDirectoriesPage)
 			switch (importWithDirectoriesPage.getWizardSelection()) {
 			case GitSelectWizardPage.EXISTING_PROJECTS_WIZARD:
 				projectsImportPage.setProjectsList(importWithDirectoriesPage
@@ -150,24 +156,30 @@ public class GitImportWizard extends AbstractGitCloneWizard implements IImportWi
 				return createGeneralProjectPage;
 
 			}
-		} else if (page == createGeneralProjectPage
-				|| page == projectsImportPage) {
+		else if (page == createGeneralProjectPage
+				|| page == projectsImportPage)
 			return null;
-		}
 		return super.getNextPage(page);
 	}
 
-	private Repository getClonedRepository() {
-		 try {
-			return  org.eclipse.egit.core.Activator
-				.getDefault().getRepositoryCache().lookupRepository(new File(cloneDestination.getDestinationFile(), Constants.DOT_GIT));
-		} catch (IOException e) {
-			Activator.error("Error looking up repository at " + cloneDestination.getDestinationFile(), e); //$NON-NLS-1$
-			return null;
-		}
+	private Repository getTargetRepository() {
+		if (existingRepo != null)
+			return existingRepo;
+		else
+			try {
+				return org.eclipse.egit.core.Activator
+						.getDefault()
+						.getRepositoryCache()
+						.lookupRepository(
+								new File(cloneDestination.getDestinationFile(),
+										Constants.DOT_GIT));
+			} catch (IOException e) {
+				Activator
+						.error("Error looking up repository at " + cloneDestination.getDestinationFile(), e); //$NON-NLS-1$
+				return null;
+			}
 	}
 
-
 	@Override
 	public boolean performFinish() {
 		try {
@@ -219,7 +231,7 @@ public class GitImportWizard extends AbstractGitCloneWizard implements IImportWi
 					IWorkingSet[] workingSetArray = projectsImportPage
 							.getSelectedWorkingSets();
 					workingSets.addAll(Arrays.asList(workingSetArray));
-					repository[0] = getClonedRepository();
+					repository[0] = getTargetRepository();
 				}
 			});
 			ProjectUtils.createProjects(projectsToCreate, repository[0],
@@ -231,7 +243,7 @@ public class GitImportWizard extends AbstractGitCloneWizard implements IImportWi
 			final File[] repoDir = new File[1];
 			PlatformUI.getWorkbench().getDisplay().syncExec(new Runnable() {
 				public void run() {
-					repoDir[0] = getClonedRepository().getDirectory();
+					repoDir[0] = getTargetRepository().getDirectory();
 				}
 			});
 			final List<IProject> previousProjects = Arrays
@@ -248,13 +260,12 @@ public class GitImportWizard extends AbstractGitCloneWizard implements IImportWi
 						throws CoreException {
 					IProject[] currentProjects = ResourcesPlugin.getWorkspace()
 							.getRoot().getProjects();
-					for (IProject current : currentProjects) {
+					for (IProject current : currentProjects)
 						if (!previousProjects.contains(current)) {
 							ConnectProviderOperation cpo = new ConnectProviderOperation(
 									current, repoDir[0]);
 							cpo.execute(actMonitor);
 						}
-					}
 				}
 			};
 			try {
@@ -276,7 +287,7 @@ public class GitImportWizard extends AbstractGitCloneWizard implements IImportWi
 					defaultLocation[0] = createGeneralProjectPage
 							.isDefaultLocation();
 					path[0] = importWithDirectoriesPage.getPath();
-					repoDir[0] = getClonedRepository().getDirectory();
+					repoDir[0] = getTargetRepository().getDirectory();
 				}
 			});
 			try {
