@@ -8,6 +8,7 @@
  *  Contributors:
  *    Kevin Sawicki (GitHub Inc.) - initial API and implementation
  *    Daniel Megert <daniel_megert@ch.ibm.com> - Added context menu to the Commit Editor's header text
+ *    Tomasz Zarna <Tomasz.Zarna@pl.ibm.com> - Add "Revert" action to Commit Editor
  *******************************************************************************/
 package org.eclipse.egit.ui.internal.commit;
 
@@ -20,6 +21,7 @@ import org.eclipse.egit.ui.internal.commit.command.CheckoutHandler;
 import org.eclipse.egit.ui.internal.commit.command.CreateBranchHandler;
 import org.eclipse.egit.ui.internal.commit.command.CreateTagHandler;
 import org.eclipse.egit.ui.internal.commit.command.CherryPickHandler;
+import org.eclipse.egit.ui.internal.commit.command.RevertHandler;
 import org.eclipse.egit.ui.internal.repository.RepositoriesView;
 import org.eclipse.jface.action.ContributionManager;
 import org.eclipse.jface.action.ControlContribution;
@@ -190,6 +192,7 @@ public class CommitEditor extends SharedHeaderFormEditor implements
 		toolbar.add(createCommandContributionItem(CreateBranchHandler.ID));
 		toolbar.add(createCommandContributionItem(CheckoutHandler.ID));
 		toolbar.add(createCommandContributionItem(CherryPickHandler.ID));
+		toolbar.add(createCommandContributionItem(RevertHandler.ID));
 		addContributions(toolbar);
 		toolbar.update(true);
 		getSite().setSelectionProvider(new ISelectionProvider() {
@@ -214,6 +217,15 @@ public class CommitEditor extends SharedHeaderFormEditor implements
 		});
 	}
 
+	/**
+	 * @see org.eclipse.ui.forms.editor.SharedHeaderFormEditor#setFocus()
+	 * @since 2.0
+	 */
+	@Override
+	public void setFocus() {
+		commitPage.getPartControl().setFocus();
+	}
+
 	private void addContributions(IToolBarManager toolBarManager) {
 		IMenuService menuService = (IMenuService) getSite().getService(
 				IMenuService.class);
@@ -246,10 +258,9 @@ public class CommitEditor extends SharedHeaderFormEditor implements
 	 */
 	public void init(IEditorSite site, IEditorInput input)
 			throws PartInitException {
-		if (input.getAdapter(RepositoryCommit.class) == null) {
+		if (input.getAdapter(RepositoryCommit.class) == null)
 			throw new PartInitException(
 					"Input could not be adapted to commit object"); //$NON-NLS-1$
-		}
 		super.init(site, input);
 		setPartName(input.getName());
 		setTitleToolTip(input.getToolTipText());
