@@ -21,11 +21,17 @@ import java.util.Map;
 import java.util.regex.Pattern;
 
 import org.eclipse.core.runtime.IAdaptable;
+import org.eclipse.core.runtime.IProgressMonitor;
+import org.eclipse.core.runtime.IStatus;
 import org.eclipse.core.runtime.Platform;
 import org.eclipse.core.runtime.PlatformObject;
+import org.eclipse.core.runtime.Status;
+import org.eclipse.core.runtime.jobs.ISchedulingRule;
+import org.eclipse.core.runtime.jobs.Job;
 import org.eclipse.egit.ui.Activator;
 import org.eclipse.egit.ui.UIIcons;
 import org.eclipse.egit.ui.UIText;
+import org.eclipse.egit.ui.UIUtils;
 import org.eclipse.egit.ui.internal.GitLabelProvider;
 import org.eclipse.egit.ui.internal.dialogs.SpellcheckableMessageArea;
 import org.eclipse.egit.ui.internal.history.CommitFileDiffViewer;
@@ -47,14 +53,13 @@ import org.eclipse.jgit.revwalk.RevCommit;
 import org.eclipse.jgit.revwalk.RevWalk;
 import org.eclipse.swt.SWT;
 import org.eclipse.swt.custom.CLabel;
-import org.eclipse.swt.custom.StackLayout;
 import org.eclipse.swt.events.DisposeEvent;
 import org.eclipse.swt.events.DisposeListener;
 import org.eclipse.swt.graphics.Image;
 import org.eclipse.swt.layout.FillLayout;
 import org.eclipse.swt.layout.GridData;
-import org.eclipse.swt.layout.GridLayout;
 import org.eclipse.swt.widgets.Composite;
+import org.eclipse.swt.widgets.Control;
 import org.eclipse.swt.widgets.Label;
 import org.eclipse.swt.widgets.Text;
 import org.eclipse.ui.PartInitException;
@@ -69,13 +74,14 @@ import org.eclipse.ui.forms.events.HyperlinkEvent;
 import org.eclipse.ui.forms.widgets.ExpandableComposite;
 import org.eclipse.ui.forms.widgets.FormToolkit;
 import org.eclipse.ui.forms.widgets.Hyperlink;
+import org.eclipse.ui.forms.widgets.ScrolledForm;
 import org.eclipse.ui.forms.widgets.Section;
 
 /**
  * Commit editor page class displaying author, committer, parent commits,
  * message, and file information in form sections.
  */
-public class CommitEditorPage extends FormPage {
+public class CommitEditorPage extends FormPage implements ISchedulingRule {
 
 	private static final String SIGNED_OFF_BY = "Signed-off-by: {0} <{1}>"; //$NON-NLS-1$
 
@@ -93,6 +99,10 @@ public class CommitEditorPage extends FormPage {
 
 	private TableViewer branchViewer;
 
+	private Section diffSection;
+
+	private CommitFileDiffViewer diffViewer;
+
 	/**
 	 * Create commit editor page
 	 *
@@ -289,26 +299,30 @@ public class CommitEditorPage extends FormPage {
 		toolkit.createLabel(tagArea, UIText.CommitEditorPage_LabelTags)
 				.setForeground(
 						toolkit.getColors().getColor(IFormColors.TB_TOGGLE));
-		fillTags(tagArea, toolkit);
+		tagLabelArea = toolkit.createComposite(tagArea);
+		GridDataFactory.fillDefaults().grab(true, true).applyTo(tagLabelArea);
+		GridLayoutFactory.fillDefaults().spacing(1, 1).applyTo(tagLabelArea);
 	}
 
-	private void fillTags(Composite parent, FormToolkit toolkit) {
-		if (tagLabelArea != null)
-			tagLabelArea.dispose();
-		tagLabelArea = toolkit.createComposite(parent);
-		GridLayoutFactory.fillDefaults().spacing(1, 1).numColumns(4)
+	private void fillDiffs(FileDiff[] diffs) {
+		diffViewer.setInput(diffs);
+		diffSection.setText(MessageFormat.format(
+				UIText.CommitEditorPage_SectionFiles,
+				Integer.valueOf(diffs.length)));
+	}
+
+	private void fillTags(FormToolkit toolkit, List<Ref> tags) {
+		for (Control child : tagLabelArea.getChildren())
+			child.dispose();
+
+		GridLayoutFactory.fillDefaults().spacing(1, 1).numColumns(tags.size())
 				.applyTo(tagLabelArea);
-		GridDataFactory.fillDefaults().grab(true, true).applyTo(tagLabelArea);
-		RevCommit commit = getCommit().getRevCommit();
-		Repository repository = getCommit().getRepository();
-		for (Ref tag : getTags()) {
-			tag = repository.peel(tag);
+
+		for (Ref tag : tags) {
 			ObjectId id = tag.getPeeledObjectId();
 			boolean annotated = id != null;
 			if (id == null)
 				id = tag.getObjectId();
-			if (!commit.equals(id))
-				continue;
 			CLabel tagLabel = new CLabel(tagLabelArea, SWT.NONE);
 			toolkit.adapt(tagLabel, false, false);
 			if (annotated)
@@ -366,6 +380,7 @@ public class CommitEditorPage extends FormPage {
 	private void createBranchesArea(Composite parent, FormToolkit toolkit,
 			int span) {
 		branchSection = createSection(parent, toolkit, span);
+		branchSection.setText(UIText.CommitEditorPage_SectionBranchesEmpty);
 		Composite branchesArea = createSectionClient(branchSection, toolkit);
 
 		branchViewer = new TableViewer(toolkit.createTable(branchesArea,
@@ -384,34 +399,10 @@ public class CommitEditorPage extends FormPage {
 		branchViewer.getTable().setData(FormToolkit.KEY_DRAW_BORDER,
 				FormToolkit.TREE_BORDER);
 
-		fillBranches();
-
 		updateSectionClient(branchSection, branchesArea, toolkit);
 	}
 
-	private void fillBranches() {
-		Repository repository = getCommit().getRepository();
-		RevCommit commit = getCommit().getRevCommit();
-		RevWalk revWalk = new RevWalk(repository);
-		List<Ref> result = new ArrayList<Ref>();
-		try {
-			Map<String, Ref> refsMap = new HashMap<String, Ref>();
-			refsMap.putAll(repository.getRefDatabase().getRefs(
-					Constants.R_HEADS));
-			refsMap.putAll(repository.getRefDatabase().getRefs(
-					Constants.R_REMOTES));
-			for (Ref ref : refsMap.values()) {
-				if (ref.isSymbolic())
-					continue;
-				RevCommit headCommit = revWalk.parseCommit(ref.getObjectId());
-				RevCommit base = revWalk.parseCommit(commit);
-				if (revWalk.isMergedInto(base, headCommit))
-					result.add(ref);
-			}
-		} catch (IOException ignored) {
-			// Ignored
-		}
-
+	private void fillBranches(List<Ref> result) {
 		branchViewer.setInput(result);
 		branchSection.setText(MessageFormat.format(
 				UIText.CommitEditorPage_SectionBranches,
@@ -419,38 +410,21 @@ public class CommitEditorPage extends FormPage {
 	}
 
 	private void createFilesArea(Composite parent, FormToolkit toolkit, int span) {
-		Section files = createSection(parent, toolkit, span);
-		Composite filesArea = createSectionClient(files, toolkit);
-		GridLayout filesAreaLayout = (GridLayout) filesArea.getLayout();
-		filesAreaLayout.marginLeft = 0;
-		filesAreaLayout.marginRight = 0;
-		filesAreaLayout.marginTop = 0;
-		filesAreaLayout.marginBottom = 0;
+		diffSection = createSection(parent, toolkit, span);
+		diffSection.setText(UIText.CommitEditorPage_SectionFilesEmpty);
+		Composite filesArea = createSectionClient(diffSection, toolkit);
 
-		CommitFileDiffViewer viewer = new CommitFileDiffViewer(filesArea,
-				getSite(), SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL
-						| SWT.FULL_SELECTION | toolkit.getBorderStyle());
-		// commit file diff viewer uses a nested composite with a stack layout
-		// and so margins need to be applied to have form toolkit style borders
-		toolkit.paintBordersFor(viewer.getTable().getParent());
-		viewer.getTable().setData(FormToolkit.KEY_DRAW_BORDER,
+		diffViewer = new CommitFileDiffViewer(filesArea, getSite(), SWT.MULTI
+				| SWT.H_SCROLL | SWT.V_SCROLL | SWT.FULL_SELECTION
+				| toolkit.getBorderStyle());
+		diffViewer.getTable().setData(FormToolkit.KEY_DRAW_BORDER,
 				FormToolkit.TREE_BORDER);
-		StackLayout viewerLayout = (StackLayout) viewer.getControl()
-				.getParent().getLayout();
-		viewerLayout.marginHeight = 2;
-		viewerLayout.marginWidth = 2;
 		GridDataFactory.fillDefaults().grab(true, true).hint(SWT.DEFAULT, 80)
-				.applyTo(viewer.getTable().getParent());
-		viewer.setContentProvider(ArrayContentProvider.getInstance());
-		viewer.setTreeWalk(getCommit().getRepository(), null);
+				.applyTo(diffViewer.getControl());
+		diffViewer.setContentProvider(ArrayContentProvider.getInstance());
+		diffViewer.setTreeWalk(getCommit().getRepository(), null);
 
-		FileDiff[] diffs = getCommit().getDiffs();
-		viewer.setInput(diffs);
-		files.setText(MessageFormat.format(
-				UIText.CommitEditorPage_SectionFiles,
-				Integer.valueOf(diffs.length)));
-
-		updateSectionClient(files, filesArea, toolkit);
+		updateSectionClient(diffSection, filesArea, toolkit);
 	}
 
 	private RepositoryCommit getCommit() {
@@ -483,15 +457,98 @@ public class CommitEditorPage extends FormPage {
 		createMessageArea(displayArea, toolkit, 2);
 		createFilesArea(displayArea, toolkit, 1);
 		createBranchesArea(displayArea, toolkit, 1);
+
+		loadSections();
+	}
+
+	private List<Ref> loadTags() {
+		RepositoryCommit repoCommit = getCommit();
+		RevCommit commit = repoCommit.getRevCommit();
+		Repository repository = repoCommit.getRepository();
+		List<Ref> tags = new ArrayList<Ref>();
+		for (Ref tag : getTags()) {
+			tag = repository.peel(tag);
+			ObjectId id = tag.getPeeledObjectId();
+			if (id == null)
+				id = tag.getObjectId();
+			if (!commit.equals(id))
+				continue;
+			tags.add(tag);
+		}
+		return tags;
+	}
+
+	private List<Ref> loadBranches() {
+		Repository repository = getCommit().getRepository();
+		RevCommit commit = getCommit().getRevCommit();
+		RevWalk revWalk = new RevWalk(repository);
+		List<Ref> result = new ArrayList<Ref>();
+		try {
+			Map<String, Ref> refsMap = new HashMap<String, Ref>();
+			refsMap.putAll(repository.getRefDatabase().getRefs(
+					Constants.R_HEADS));
+			refsMap.putAll(repository.getRefDatabase().getRefs(
+					Constants.R_REMOTES));
+			for (Ref ref : refsMap.values()) {
+				if (ref.isSymbolic())
+					continue;
+				RevCommit headCommit = revWalk.parseCommit(ref.getObjectId());
+				RevCommit base = revWalk.parseCommit(commit);
+				if (revWalk.isMergedInto(base, headCommit))
+					result.add(ref);
+			}
+		} catch (IOException ignored) {
+			// Ignored
+		}
+		return result;
+	}
+
+	private void loadSections() {
+		RepositoryCommit commit = getCommit();
+		Job refreshJob = new Job(MessageFormat.format(
+				UIText.CommitEditorPage_JobName, commit.getRevCommit().name())) {
+
+			@Override
+			protected IStatus run(IProgressMonitor monitor) {
+				final List<Ref> tags = loadTags();
+				final List<Ref> branches = loadBranches();
+				final FileDiff[] diffs = getCommit().getDiffs();
+
+				final ScrolledForm form = getManagedForm().getForm();
+				if (UIUtils.isUsable(form))
+					form.getDisplay().syncExec(new Runnable() {
+
+						public void run() {
+							if (!UIUtils.isUsable(form))
+								return;
+
+							fillTags(getManagedForm().getToolkit(), tags);
+							fillDiffs(diffs);
+							fillBranches(branches);
+							form.layout(true, true);
+						}
+					});
+
+				return Status.OK_STATUS;
+			}
+		};
+		refreshJob.setRule(this);
+		refreshJob.schedule();
 	}
 
 	/**
 	 * Refresh the editor page
 	 */
 	public void refresh() {
-		fillTags(tagLabelArea.getParent(), getManagedForm().getToolkit());
-		fillBranches();
-		getManagedForm().getForm().layout(true, true);
+		loadSections();
+	}
+
+	public boolean contains(ISchedulingRule rule) {
+		return rule == this;
+	}
+
+	public boolean isConflicting(ISchedulingRule rule) {
+		return rule == this;
 	}
 
 }
