@@ -283,10 +283,9 @@ public class RepositorySelectionPage extends WizardPage implements IRepositorySe
 		 * @return protocol handling this URI
 		 */
 		public static Protocol fromUri(URIish uri) {
-			for (Protocol p : protocols.values()) {
+			for (Protocol p : protocols.values())
 				if (p.handles(uri))
 					return p;
-			}
 			return null;
 		}
 	}
@@ -332,11 +331,10 @@ public class RepositorySelectionPage extends WizardPage implements IRepositorySe
 					if (index > 0)
 						text = text.substring(0, index);
 					URIish u = new URIish(text);
-					if (canHandleProtocol(u)) {
+					if (canHandleProtocol(u))
 						if (Protocol.GIT.handles(u) || Protocol.SSH.handles(u)
 								|| text.endsWith(Constants.DOT_GIT_EXT))
 							preset = text;
-					}
 				}
 			} catch (URISyntaxException e) {
 				// ignore, preset is null
@@ -522,7 +520,7 @@ public class RepositorySelectionPage extends WizardPage implements IRepositorySe
 				DirectoryDialog dialog = new DirectoryDialog(getShell());
 				// if a file was selected before, let's try to open
 				// the directory dialog on the same directory
-				if (!uriText.getText().equals(EMPTY_STRING)) {
+				if (!uriText.getText().equals(EMPTY_STRING))
 					try {
 						// first we try if this is a simple file name
 						File testFile = new File(uriText.getText());
@@ -545,7 +543,6 @@ public class RepositorySelectionPage extends WizardPage implements IRepositorySe
 						// ignore here, we just' don't set the directory in the
 						// browser
 					}
-				}
 				// if nothing else, we start the search from the default folder for repositories
 				if (EMPTY_STRING.equals(dialog.getFilterPath()))
 					dialog.setFilterPath(Activator.getDefault().getPreferenceStore().getString(UIPreferences.DEFAULT_REPO_DIR));
@@ -624,9 +621,8 @@ public class RepositorySelectionPage extends WizardPage implements IRepositorySe
 
 		newLabel(g, UIText.RepositorySelectionPage_promptScheme + ":"); //$NON-NLS-1$
 		scheme = new Combo(g, SWT.DROP_DOWN | SWT.READ_ONLY);
-		for (Protocol p : Protocol.values()) {
+		for (Protocol p : Protocol.values())
 			scheme.add(p.getDefaultScheme());
-		}
 		scheme.addSelectionListener(new SelectionAdapter() {
 			public void widgetSelected(final SelectionEvent e) {
 				final int idx = scheme.getSelectionIndex();
@@ -658,13 +654,12 @@ public class RepositorySelectionPage extends WizardPage implements IRepositorySe
 				final String val = nullString(portText.getText());
 				if (val == null)
 					setURI(uri.setPort(-1));
-				else {
+				else
 					try {
 						setURI(uri.setPort(Integer.parseInt(val)));
 					} catch (NumberFormatException err) {
 						// Ignore it for now.
 					}
-				}
 			}
 		});
 	}
@@ -751,14 +746,13 @@ public class RepositorySelectionPage extends WizardPage implements IRepositorySe
 		sb.append(": "); //$NON-NLS-1$
 		boolean first = true;
 		List<URIish> uris;
-		if (sourceSelection) {
+		if (sourceSelection)
 			uris = rc.getURIs();
-		} else {
+		else {
 			uris = rc.getPushURIs();
 			// if no push URIs are defined, use fetch URIs instead
-			if (uris.isEmpty()) {
+			if (uris.isEmpty())
 				uris = rc.getURIs();
-			}
 		}
 
 		for (final URIish u : uris) {
@@ -1014,7 +1008,7 @@ public class RepositorySelectionPage extends WizardPage implements IRepositorySe
 	public GitRepositoryInfo getGitRepositoryInfo() {
 		GitRepositoryInfo info = new GitRepositoryInfo(uri.toString());
 		info.setCredentials(user, password);
-		info.setShouldSaveCredentialsInSecureStore(true);
+		info.setShouldSaveCredentialsInSecureStore(storeInSecureStore);
 		uriProposalHandler.updateProposals();
 		return info;
 	}
