@@ -15,8 +15,6 @@ package org.eclipse.egit.ui.internal.dialogs;
 
 import java.io.File;
 import java.io.IOException;
-import java.io.StringReader;
-import java.io.StringWriter;
 import java.net.URI;
 import java.text.MessageFormat;
 import java.util.ArrayList;
@@ -24,7 +22,6 @@ import java.util.Collection;
 import java.util.Collections;
 import java.util.Comparator;
 import java.util.Iterator;
-import java.util.LinkedHashSet;
 import java.util.Set;
 
 import org.eclipse.core.resources.IFile;
@@ -38,8 +35,10 @@ import org.eclipse.egit.ui.UIPreferences;
 import org.eclipse.egit.ui.UIText;
 import org.eclipse.egit.ui.UIUtils;
 import org.eclipse.egit.ui.internal.CompareUtils;
+import org.eclipse.egit.ui.internal.commit.CommitMessageHistory;
 import org.eclipse.egit.ui.internal.commit.CommitProposalProcessor;
 import org.eclipse.egit.ui.internal.dialogs.CommitItem.Status;
+import org.eclipse.egit.ui.internal.dialogs.CommitMessageComponent.CommitStatus;
 import org.eclipse.jface.dialogs.IDialogConstants;
 import org.eclipse.jface.dialogs.IDialogSettings;
 import org.eclipse.jface.dialogs.IMessageProvider;
@@ -54,11 +53,7 @@ import org.eclipse.jface.resource.JFaceResources;
 import org.eclipse.jface.resource.LocalResourceManager;
 import org.eclipse.jface.resource.ResourceManager;
 import org.eclipse.jface.text.DocumentEvent;
-import org.eclipse.jface.text.IDocument;
 import org.eclipse.jface.text.IDocumentListener;
-import org.eclipse.jface.text.contentassist.ContentAssistant;
-import org.eclipse.jface.text.contentassist.IContentAssistant;
-import org.eclipse.jface.text.source.ISourceViewer;
 import org.eclipse.jface.viewers.ArrayContentProvider;
 import org.eclipse.jface.viewers.CheckStateChangedEvent;
 import org.eclipse.jface.viewers.CheckboxTableViewer;
@@ -75,7 +70,6 @@ import org.eclipse.jface.viewers.ViewerFilter;
 import org.eclipse.jface.window.Window;
 import org.eclipse.jgit.api.AddCommand;
 import org.eclipse.jgit.api.Git;
-import org.eclipse.jgit.api.errors.NoFilepatternException;
 import org.eclipse.jgit.lib.Constants;
 import org.eclipse.jgit.lib.FileMode;
 import org.eclipse.jgit.lib.IndexDiff;
@@ -107,11 +101,8 @@ import org.eclipse.swt.widgets.Text;
 import org.eclipse.swt.widgets.ToolBar;
 import org.eclipse.swt.widgets.ToolItem;
 import org.eclipse.team.core.RepositoryProvider;
-import org.eclipse.ui.IMemento;
 import org.eclipse.ui.ISharedImages;
 import org.eclipse.ui.PlatformUI;
-import org.eclipse.ui.WorkbenchException;
-import org.eclipse.ui.XMLMemento;
 import org.eclipse.ui.dialogs.PreferencesUtil;
 import org.eclipse.ui.forms.IFormColors;
 import org.eclipse.ui.forms.widgets.ExpandableComposite;
@@ -124,77 +115,10 @@ import org.eclipse.ui.forms.widgets.Section;
  */
 public class CommitDialog extends TitleAreaDialog {
 
-	private static final String KEY_MESSAGE = "message"; //$NON-NLS-1$
-
-	private static final String KEY_MESSAGES = "messages"; //$NON-NLS-1$
-
 	private static IPreferenceStore getPreferenceStore() {
 		return org.eclipse.egit.ui.Activator.getDefault().getPreferenceStore();
 	}
 
-	private static Set<String> getCommitHistory() {
-		String all = getPreferenceStore().getString(
-				UIPreferences.COMMIT_DIALOG_HISTORY_MESSAGES);
-		if (all.length() == 0)
-			return Collections.emptySet();
-		int max = getCommitHistorySize();
-		if (max < 1)
-			return Collections.emptySet();
-		XMLMemento memento;
-		try {
-			memento = XMLMemento.createReadRoot(new StringReader(all));
-		} catch (WorkbenchException e) {
-			org.eclipse.egit.ui.Activator.logError(
-					"Error reading commit message history", e); //$NON-NLS-1$
-			return Collections.emptySet();
-		}
-		Set<String> messages = new LinkedHashSet<String>();
-		for (IMemento child : memento.getChildren(KEY_MESSAGE)) {
-			messages.add(child.getTextData());
-			if (messages.size() == max)
-				break;
-		}
-		return messages;
-	}
-
-	private static void saveCommitHistory(String message) {
-		if (message == null || message.length() == 0)
-			return;
-		int size = getCommitHistorySize();
-		if (size < 1)
-			return;
-
-		XMLMemento memento = XMLMemento.createWriteRoot(KEY_MESSAGES);
-		memento.createChild(KEY_MESSAGE).putTextData(message);
-
-		int count = 1;
-		if (count < size) {
-			Set<String> history = getCommitHistory();
-			history.remove(message);
-			for (String previous : history) {
-				memento.createChild(KEY_MESSAGE).putTextData(previous);
-				count++;
-				if (count == size)
-					break;
-			}
-		}
-		StringWriter writer = new StringWriter();
-		try {
-			memento.save(writer);
-			getPreferenceStore().setValue(
-					UIPreferences.COMMIT_DIALOG_HISTORY_MESSAGES,
-					writer.toString());
-		} catch (IOException e) {
-			org.eclipse.egit.ui.Activator.logError(
-					"Error writing commit message history", e); //$NON-NLS-1$
-		}
-	}
-
-	private static int getCommitHistorySize() {
-		return getPreferenceStore().getInt(
-				UIPreferences.COMMIT_DIALOG_HISTORY_SIZE);
-	}
-
 	static class CommitStatusLabelProvider extends ColumnLabelProvider {
 
 		private Image DEFAULT = PlatformUI.getWorkbench().getSharedImages()
@@ -688,30 +612,22 @@ public class CommitDialog extends TitleAreaDialog {
 
 		messageSection.setTextClient(headerArea);
 
-		commitText = new SpellcheckableMessageArea(messageArea, commitMessage,
-				SWT.NONE) {
-
-			protected IContentAssistant createContentAssistant(
-					ISourceViewer viewer) {
-				ContentAssistant assistant = new ContentAssistant();
-				assistant.enableAutoInsert(true);
-				Collection<String> paths = getFileList();
-				Collection<String> messages = getCommitHistory();
-				final CommitProposalProcessor processor = new CommitProposalProcessor(
-						messages.toArray(new String[messages.size()]),
-						paths.toArray(new String[paths.size()]));
-				viewer.getTextWidget().addDisposeListener(
-						new DisposeListener() {
-
-							public void widgetDisposed(DisposeEvent e) {
-								processor.dispose();
-							}
-						});
-				assistant.setContentAssistProcessor(processor,
-						IDocument.DEFAULT_CONTENT_TYPE);
-				return assistant;
+		final CommitProposalProcessor commitProposalProcessor = new CommitProposalProcessor() {
+			@Override
+			protected Collection<String> computeFileNameProposals() {
+				return getFileList();
 			}
 
+			@Override
+			protected Collection<String> computeMessageProposals() {
+				return CommitMessageHistory.getCommitHistory();
+			}
+		};
+		commitText = new CommitMessageArea(messageArea, commitMessage, SWT.NONE) {
+			@Override
+			protected CommitProposalProcessor getCommitProposalProcessor() {
+				return commitProposalProcessor;
+			}
 		};
 		commitText
 				.setData(FormToolkit.KEY_DRAW_BORDER, FormToolkit.TEXT_BORDER);
@@ -1009,12 +925,26 @@ public class CommitDialog extends TitleAreaDialog {
 	}
 
 	private void updateMessage() {
-		String message = commitMessageComponent.getMessage();
-		if (message == null && filesViewer.getCheckedElements().length == 0
-				&& !amendingItem.getSelection())
+		String message = null;
+		int type = IMessageProvider.NONE;
+
+		String commitMsg = commitMessageComponent.getCommitMessage();
+		if (commitMsg == null || commitMsg.trim().length() == 0) {
+			message = UIText.CommitDialog_Message;
+			type = IMessageProvider.INFORMATION;
+		} else if (filesViewer.getCheckedElements().length == 0
+				&& !amendingItem.getSelection()) {
 			message = UIText.CommitDialog_MessageNoFilesSelected;
-		setMessage(message, IMessageProvider.INFORMATION);
-		commitButton.setEnabled(message == null);
+			type = IMessageProvider.INFORMATION;
+		} else {
+			CommitStatus status = commitMessageComponent.getStatus();
+			message = status.getMessage();
+			type = status.getMessageType();
+		}
+
+		setMessage(message, type);
+		commitButton.setEnabled(type == IMessageProvider.WARNING
+				|| type == IMessageProvider.NONE);
 	}
 
 	private Collection<String> getFileList() {
@@ -1057,7 +987,7 @@ public class CommitDialog extends TitleAreaDialog {
 				}
 				try {
 					addCommand.call();
-				} catch (NoFilepatternException e) {
+				} catch (Exception e) {
 					Activator.logError(UIText.CommitDialog_ErrorAddingFiles, e);
 				}
 				for (Iterator<?> it = sel.iterator(); it.hasNext();) {
@@ -1153,7 +1083,7 @@ public class CommitDialog extends TitleAreaDialog {
 		IDialogSettings settings = org.eclipse.egit.ui.Activator
 			.getDefault().getDialogSettings();
 		settings.put(SHOW_UNTRACKED_PREF, showUntracked);
-		saveCommitHistory(getCommitMessage());
+		CommitMessageHistory.saveCommitHistory(getCommitMessage());
 		super.okPressed();
 	}
 
