@@ -1,7 +1,7 @@
 /*******************************************************************************
  * Copyright (C) 2010, Benjamin Muskalla <bmuskalla@eclipsesource.com>
  * Copyright (C) 2011, Matthias Sohn <matthias.sohn@sap.com>
- * Copyright (C) 2011, IBM Corporation
+ * Copyright (C) 2011-2012, IBM Corporation
  *
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
@@ -10,6 +10,7 @@
  *
  * Contributors:
  *    Benjamin Muskalla (EclipseSource) - initial implementation
+ *    Tomasz Zarna (IBM) - show whitespace action, bug 371353
  *******************************************************************************/
 package org.eclipse.egit.ui.internal.dialogs;
 
@@ -19,6 +20,7 @@ import java.util.LinkedList;
 import java.util.List;
 import java.util.Map;
 
+import org.eclipse.core.runtime.Assert;
 import org.eclipse.core.runtime.IAdaptable;
 import org.eclipse.egit.core.internal.Utils;
 import org.eclipse.egit.ui.Activator;
@@ -33,15 +35,19 @@ import org.eclipse.jface.action.MenuManager;
 import org.eclipse.jface.action.Separator;
 import org.eclipse.jface.action.SubMenuManager;
 import org.eclipse.jface.commands.ActionHandler;
+import org.eclipse.jface.preference.IPreferenceStore;
 import org.eclipse.jface.resource.ImageDescriptor;
 import org.eclipse.jface.text.Document;
-import org.eclipse.jface.text.ITextListener;
 import org.eclipse.jface.text.IDocument;
+import org.eclipse.jface.text.IPainter;
+import org.eclipse.jface.text.ITextListener;
 import org.eclipse.jface.text.ITextOperationTarget;
 import org.eclipse.jface.text.ITextViewer;
+import org.eclipse.jface.text.ITextViewerExtension2;
 import org.eclipse.jface.text.MarginPainter;
 import org.eclipse.jface.text.Position;
 import org.eclipse.jface.text.TextEvent;
+import org.eclipse.jface.text.WhitespaceCharacterPainter;
 import org.eclipse.jface.text.contentassist.ICompletionProposal;
 import org.eclipse.jface.text.contentassist.IContentAssistant;
 import org.eclipse.jface.text.quickassist.IQuickAssistInvocationContext;
@@ -54,6 +60,8 @@ import org.eclipse.jface.text.source.IAnnotationModel;
 import org.eclipse.jface.text.source.ISharedTextColors;
 import org.eclipse.jface.text.source.ISourceViewer;
 import org.eclipse.jface.text.source.SourceViewer;
+import org.eclipse.jface.util.IPropertyChangeListener;
+import org.eclipse.jface.util.PropertyChangeEvent;
 import org.eclipse.jface.viewers.ISelectionChangedListener;
 import org.eclipse.jface.viewers.SelectionChangedEvent;
 import org.eclipse.swt.SWT;
@@ -77,6 +85,7 @@ import org.eclipse.ui.editors.text.EditorsUI;
 import org.eclipse.ui.editors.text.TextSourceViewerConfiguration;
 import org.eclipse.ui.handlers.IHandlerActivation;
 import org.eclipse.ui.handlers.IHandlerService;
+import org.eclipse.ui.texteditor.AbstractTextEditor;
 import org.eclipse.ui.texteditor.AnnotationPreference;
 import org.eclipse.ui.texteditor.DefaultMarkerAnnotationAccess;
 import org.eclipse.ui.texteditor.ITextEditorActionDefinitionIds;
@@ -123,19 +132,78 @@ public class SpellcheckableMessageArea extends Composite {
 			boolean isEnabled= (fOperationTarget != null && fOperationTarget.canDoOperation(fOperationCode));
 			setEnabled(isEnabled);
 
-			if (wasEnabled != isEnabled) {
+			if (wasEnabled != isEnabled)
 				firePropertyChange(ENABLED, wasEnabled ? Boolean.TRUE : Boolean.FALSE, isEnabled ? Boolean.TRUE : Boolean.FALSE);
-			}
 		}
 
 		/**
 		 * @see Action#run()
 		 */
 		public void run() {
-			if (fOperationCode != -1 && fOperationTarget != null) {
+			if (fOperationCode != -1 && fOperationTarget != null)
 				fOperationTarget.doOperation(fOperationCode);
+		}
+	}
+
+	private static abstract class TextEditorPropertyAction extends Action implements IPropertyChangeListener {
+
+		private SourceViewer viewer;
+		private String preferenceKey;
+		private IPreferenceStore store;
+
+		public TextEditorPropertyAction(String label, SourceViewer viewer, String preferenceKey) {
+			super(label, IAction.AS_CHECK_BOX);
+			this.viewer = viewer;
+			this.preferenceKey = preferenceKey;
+			this.store = EditorsUI.getPreferenceStore();
+			if (store != null)
+				store.addPropertyChangeListener(this);
+			synchronizeWithPreference();
+		}
+
+		public void propertyChange(PropertyChangeEvent event) {
+			if (event.getProperty().equals(getPreferenceKey()))
+				synchronizeWithPreference();
+		}
+
+		protected void synchronizeWithPreference() {
+			boolean checked = false;
+			if (store != null)
+				checked = store.getBoolean(getPreferenceKey());
+			if (checked != isChecked()) {
+				setChecked(checked);
+				toggleState(checked);
+			} else if (checked) {
+				toggleState(false);
+				toggleState(true);
 			}
 		}
+
+		private String getPreferenceKey() {
+			return preferenceKey;
+		}
+
+		@Override
+		public void run() {
+			toggleState(isChecked());
+			if (store != null)
+				store.setValue(getPreferenceKey(), isChecked());
+		}
+
+		public void dispose() {
+			if (store != null)
+				store.removePropertyChangeListener(this);
+		}
+
+		/**
+		 * @param checked
+		 *            new state
+		 */
+		abstract protected void toggleState(boolean checked);
+
+		protected ITextViewer getTextViewer() {
+			return viewer;
+		}
 	}
 
 	private final SourceViewer sourceViewer;
@@ -191,7 +259,15 @@ public class SpellcheckableMessageArea extends Composite {
 		configureHardWrap();
 
 		final SourceViewerDecorationSupport support = configureAnnotationPreferences();
-		final IHandlerActivation handlerActivation = installQuickFixActionHandler();
+		if (isEditable(sourceViewer)) {
+			final IHandlerActivation handlerActivation = installQuickFixActionHandler();
+			getTextWidget().addDisposeListener(new DisposeListener() {
+
+				public void widgetDisposed(DisposeEvent e) {
+					getHandlerService().deactivateHandler(handlerActivation);
+				}
+			});
+		}
 
 		Document document = new Document(initialText);
 
@@ -236,7 +312,6 @@ public class SpellcheckableMessageArea extends Composite {
 		getTextWidget().addDisposeListener(new DisposeListener() {
 			public void widgetDisposed(DisposeEvent disposeEvent) {
 				support.uninstall();
-				getHandlerService().deactivateHandler(handlerActivation);
 			}
 		});
 	}
@@ -255,68 +330,133 @@ public class SpellcheckableMessageArea extends Composite {
 					private boolean active = true;
 
 					public void modifyText(ModifyEvent e) {
-						if (!active) {
+						if (!active)
 							return;
-						}
 						String lineDelimiter = textWidget.getLineDelimiter();
 						List<WrapEdit> wrapEdits = calculateWrapEdits(
 								textWidget.getText(), MAX_LINE_WIDTH,
 								lineDelimiter);
 						// Prevent infinite loop because replaceTextRange causes a ModifyEvent
 						active = false;
-						for (WrapEdit wrapEdit : wrapEdits) {
+						for (WrapEdit wrapEdit : wrapEdits)
 							textWidget.replaceTextRange(wrapEdit.getStart(), wrapEdit.getLength(), lineDelimiter);
-						}
 						active = true;
 					}
 				};
 				textWidget.addModifyListener(hardWrapModifyListener);
 			}
-		} else {
-			if (hardWrapModifyListener != null) {
-				getTextWidget().removeModifyListener(hardWrapModifyListener);
-				hardWrapModifyListener = null;
-			}
+		} else if (hardWrapModifyListener != null) {
+			getTextWidget().removeModifyListener(hardWrapModifyListener);
+			hardWrapModifyListener = null;
 		}
 	}
 
 	private void configureContextMenu() {
-		final TextViewerAction cutAction = new TextViewerAction(sourceViewer, ITextOperationTarget.CUT);
-		cutAction.setText(UIText.SpellCheckingMessageArea_cut);
-		cutAction.setActionDefinitionId(IWorkbenchCommandConstants.EDIT_CUT);
+		final boolean editable = isEditable(sourceViewer);
+		final TextViewerAction cutAction;
+		final TextViewerAction undoAction;
+		final TextViewerAction redoAction;
+		final TextViewerAction pasteAction;
+		if (editable) {
+			cutAction = new TextViewerAction(sourceViewer,
+					ITextOperationTarget.CUT);
+			cutAction.setText(UIText.SpellCheckingMessageArea_cut);
+			cutAction
+					.setActionDefinitionId(IWorkbenchCommandConstants.EDIT_CUT);
 
-		final TextViewerAction copyAction = new TextViewerAction(sourceViewer, ITextOperationTarget.COPY);
+			undoAction = new TextViewerAction(sourceViewer,
+					ITextOperationTarget.UNDO);
+			undoAction.setText(UIText.SpellcheckableMessageArea_undo);
+			undoAction
+					.setActionDefinitionId(IWorkbenchCommandConstants.EDIT_UNDO);
+
+			redoAction = new TextViewerAction(sourceViewer,
+					ITextOperationTarget.REDO);
+			redoAction.setText(UIText.SpellcheckableMessageArea_redo);
+			redoAction
+					.setActionDefinitionId(IWorkbenchCommandConstants.EDIT_REDO);
+
+			pasteAction = new TextViewerAction(sourceViewer,
+					ITextOperationTarget.PASTE);
+			pasteAction.setText(UIText.SpellCheckingMessageArea_paste);
+			pasteAction
+					.setActionDefinitionId(IWorkbenchCommandConstants.EDIT_PASTE);
+		} else {
+			cutAction = null;
+			undoAction = null;
+			redoAction = null;
+			pasteAction = null;
+		}
+
+		final TextViewerAction copyAction = new TextViewerAction(sourceViewer,
+				ITextOperationTarget.COPY);
 		copyAction.setText(UIText.SpellCheckingMessageArea_copy);
 		copyAction.setActionDefinitionId(IWorkbenchCommandConstants.EDIT_COPY);
 
-		final TextViewerAction pasteAction = new TextViewerAction(sourceViewer, ITextOperationTarget.PASTE);
-		pasteAction.setText(UIText.SpellCheckingMessageArea_paste);
-		pasteAction.setActionDefinitionId(IWorkbenchCommandConstants.EDIT_PASTE);
-
-		final TextViewerAction selectAllAction = new TextViewerAction(sourceViewer, ITextOperationTarget.SELECT_ALL);
+		final TextViewerAction selectAllAction = new TextViewerAction(
+				sourceViewer, ITextOperationTarget.SELECT_ALL);
 		selectAllAction.setText(UIText.SpellCheckingMessageArea_selectAll);
-		selectAllAction.setActionDefinitionId(IWorkbenchCommandConstants.EDIT_SELECT_ALL);
+		selectAllAction
+				.setActionDefinitionId(IWorkbenchCommandConstants.EDIT_SELECT_ALL);
 
-		final TextViewerAction undoAction = new TextViewerAction(sourceViewer,
-				ITextOperationTarget.UNDO);
-		undoAction.setText(UIText.SpellcheckableMessageArea_undo);
-		undoAction.setActionDefinitionId(IWorkbenchCommandConstants.EDIT_UNDO);
+		final TextEditorPropertyAction showWhitespaceAction = new TextEditorPropertyAction(
+				UIText.SpellcheckableMessageArea_showWhitespace,
+				sourceViewer,
+				AbstractTextEditor.PREFERENCE_SHOW_WHITESPACE_CHARACTERS) {
 
-		final TextViewerAction redoAction = new TextViewerAction(sourceViewer,
-				ITextOperationTarget.REDO);
-		redoAction.setText(UIText.SpellcheckableMessageArea_redo);
-		redoAction.setActionDefinitionId(IWorkbenchCommandConstants.EDIT_REDO);
+			private IPainter whitespaceCharPainter;
+
+			@Override
+			protected void toggleState(boolean checked) {
+				if (checked)
+					installPainter();
+				else
+					uninstallPainter();
+			}
+
+			/**
+			 * Installs the painter on the viewer.
+			 */
+			private void installPainter() {
+				Assert.isTrue(whitespaceCharPainter == null);
+				ITextViewer v = getTextViewer();
+				if (v instanceof ITextViewerExtension2) {
+					whitespaceCharPainter = new WhitespaceCharacterPainter(v);
+					((ITextViewerExtension2) v).addPainter(whitespaceCharPainter);
+				}
+			}
+
+			/**
+			 * Remove the painter from the viewer.
+			 */
+			private void uninstallPainter() {
+				if (whitespaceCharPainter == null)
+					return;
+				ITextViewer v = getTextViewer();
+				if (v instanceof ITextViewerExtension2)
+					((ITextViewerExtension2) v)
+							.removePainter(whitespaceCharPainter);
+				whitespaceCharPainter.deactivate(true);
+				whitespaceCharPainter = null;
+			}
+		};
 
 		MenuManager contextMenu = new MenuManager();
-		contextMenu.add(cutAction);
+		if (cutAction != null)
+			contextMenu.add(cutAction);
 		contextMenu.add(copyAction);
-		contextMenu.add(pasteAction);
+		if (pasteAction != null)
+			contextMenu.add(pasteAction);
 		contextMenu.add(selectAllAction);
-		contextMenu.add(undoAction);
-		contextMenu.add(redoAction);
+		if (undoAction != null)
+			contextMenu.add(undoAction);
+		if (redoAction != null)
+			contextMenu.add(redoAction);
+		contextMenu.add(new Separator());
+		contextMenu.add(showWhitespaceAction);
 		contextMenu.add(new Separator());
 
-		if(isEditable(sourceViewer)) {
+		if (editable) {
 			final SubMenuManager quickFixMenu = new SubMenuManager(contextMenu);
 			quickFixMenu.setVisible(true);
 			quickFixMenu.addMenuListener(new IMenuListener() {
@@ -326,10 +466,11 @@ public class SpellcheckableMessageArea extends Composite {
 				}
 			});
 		}
-		StyledText textWidget = getTextWidget();
-		getTextWidget().setMenu(contextMenu.createContextMenu(textWidget));
 
-		getTextWidget().addFocusListener(new FocusListener() {
+		final StyledText textWidget = getTextWidget();
+		textWidget.setMenu(contextMenu.createContextMenu(textWidget));
+
+		textWidget.addFocusListener(new FocusListener() {
 
 			private IHandlerActivation cutHandlerActivation;
 			private IHandlerActivation copyHandlerActivation;
@@ -339,41 +480,56 @@ public class SpellcheckableMessageArea extends Composite {
 			private IHandlerActivation redoHandlerActivation;
 
 			public void focusGained(FocusEvent e) {
-				cutAction.update();
+				IHandlerService service = (IHandlerService) PlatformUI
+						.getWorkbench().getService(IHandlerService.class);
+				if (cutAction != null) {
+					cutAction.update();
+					cutHandlerActivation = service.activateHandler(
+							IWorkbenchCommandConstants.EDIT_CUT,
+							new ActionHandler(cutAction),
+							new ActiveShellExpression(getParent().getShell()));
+				}
 				copyAction.update();
-				IHandlerService service = (IHandlerService) PlatformUI.getWorkbench().getService(IHandlerService.class);
-				this.cutHandlerActivation = service.activateHandler(IWorkbenchCommandConstants.EDIT_CUT, new ActionHandler(cutAction), new ActiveShellExpression(getParent().getShell()));
-	            this.copyHandlerActivation = service.activateHandler(IWorkbenchCommandConstants.EDIT_COPY, new ActionHandler(copyAction), new ActiveShellExpression(getParent().getShell()));
-	            this.pasteHandlerActivation = service.activateHandler(IWorkbenchCommandConstants.EDIT_PASTE, new ActionHandler(pasteAction), new ActiveShellExpression(getParent().getShell()));
-	            this.selectAllHandlerActivation = service.activateHandler(IWorkbenchCommandConstants.EDIT_SELECT_ALL, new ActionHandler(selectAllAction), new ActiveShellExpression(getParent().getShell()));
-				undoHandlerActivation = service.activateHandler(
-						IWorkbenchCommandConstants.EDIT_UNDO,
-						new ActionHandler(undoAction),
+
+				copyHandlerActivation = service.activateHandler(
+						IWorkbenchCommandConstants.EDIT_COPY,
+						new ActionHandler(copyAction),
 						new ActiveShellExpression(getParent().getShell()));
-				redoHandlerActivation = service.activateHandler(
-						IWorkbenchCommandConstants.EDIT_REDO,
-						new ActionHandler(redoAction),
+				if (pasteAction != null)
+					this.pasteHandlerActivation = service.activateHandler(
+							IWorkbenchCommandConstants.EDIT_PASTE,
+							new ActionHandler(pasteAction),
+							new ActiveShellExpression(getParent().getShell()));
+				selectAllHandlerActivation = service.activateHandler(
+						IWorkbenchCommandConstants.EDIT_SELECT_ALL,
+						new ActionHandler(selectAllAction),
 						new ActiveShellExpression(getParent().getShell()));
+				if (undoAction != null)
+					undoHandlerActivation = service.activateHandler(
+							IWorkbenchCommandConstants.EDIT_UNDO,
+							new ActionHandler(undoAction),
+							new ActiveShellExpression(getParent().getShell()));
+				if (redoAction != null)
+					redoHandlerActivation = service.activateHandler(
+							IWorkbenchCommandConstants.EDIT_REDO,
+							new ActionHandler(redoAction),
+							new ActiveShellExpression(getParent().getShell()));
 			}
 
 			public void focusLost(FocusEvent e) {
 				IHandlerService service = (IHandlerService) PlatformUI.getWorkbench().getService(IHandlerService.class);
 
-				if (cutHandlerActivation != null) {
+				if (cutHandlerActivation != null)
 					service.deactivateHandler(cutHandlerActivation);
-				}
 
-				if (copyHandlerActivation != null) {
+				if (copyHandlerActivation != null)
 					service.deactivateHandler(copyHandlerActivation);
-				}
 
-				if (pasteHandlerActivation != null) {
+				if (pasteHandlerActivation != null)
 					service.deactivateHandler(pasteHandlerActivation);
-				}
 
-				if (selectAllHandlerActivation != null) {
+				if (selectAllHandlerActivation != null)
 					service.deactivateHandler(selectAllHandlerActivation);
-				}
 
 				if (undoHandlerActivation != null)
 					service.deactivateHandler(undoHandlerActivation);
@@ -387,17 +543,27 @@ public class SpellcheckableMessageArea extends Composite {
         sourceViewer.addSelectionChangedListener(new ISelectionChangedListener() {
 
 					public void selectionChanged(SelectionChangedEvent event) {
-						cutAction.update();
+						if (cutAction != null)
+							cutAction.update();
 						copyAction.update();
 					}
 
-        });
+				});
 
-		sourceViewer.addTextListener(new ITextListener() {
+		if (editable)
+			sourceViewer.addTextListener(new ITextListener() {
 
-			public void textChanged(TextEvent event) {
-				undoAction.update();
-				redoAction.update();
+				public void textChanged(TextEvent event) {
+					if (undoAction != null)
+						undoAction.update();
+					if (redoAction != null)
+						redoAction.update();
+				}
+			});
+
+		textWidget.addDisposeListener(new DisposeListener() {
+			public void widgetDisposed(DisposeEvent disposeEvent) {
+				showWhitespaceAction.dispose();
 			}
 		});
 	}
@@ -441,9 +607,8 @@ public class SpellcheckableMessageArea extends Composite {
 
 			public ImageDescriptor getImageDescriptor() {
 				Image image = proposal.getImage();
-				if (image != null) {
+				if (image != null)
 					return ImageDescriptor.createFromImage(image);
-				}
 				return null;
 			}
 		};
@@ -529,11 +694,7 @@ public class SpellcheckableMessageArea extends Composite {
 	private ActionHandler createQuickFixActionHandler(
 			final ITextOperationTarget textOperationTarget) {
 		Action quickFixAction = new Action() {
-			/*
-			 * (non-Javadoc)
-			 *
-			 * @see org.eclipse.jface.action.Action#run()
-			 */
+
 			public void run() {
 				textOperationTarget.doOperation(ISourceViewer.QUICK_ASSIST);
 			}
@@ -685,9 +846,8 @@ public class SpellcheckableMessageArea extends Composite {
 				lineLength += wordLength;
 			}
 
-			if (chunkIndex != chunks.length - 1) {
+			if (chunkIndex != chunks.length - 1)
 				offset += lineDelimiterLength;
-			}
 		}
 
 		return wrapEdits;
