@@ -3,8 +3,9 @@
  * Copyright (C) 2007, Robin Rosenberg <robin.rosenberg@dewire.com>
  * Copyright (C) 2008, Roger C. Soares <rogersoares@intelinet.com.br>
  * Copyright (C) 2008, Shawn O. Pearce <spearce@spearce.org>
- * Copyright (C) 2011, Mathias Kinzler <mathias.kinzler@sap.com>
- * Copyright (C) 2011, Matthias Sohn <matthias.sohn@sap.com>
+ * Copyright (C) 2011-2012, Mathias Kinzler <mathias.kinzler@sap.com>
+ * Copyright (C) 2011-2012, Matthias Sohn <matthias.sohn@sap.com>
+ * Copyright (C) 2012, Robin Stocker <robin@nibor.org>
  *
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
@@ -36,11 +37,17 @@ import org.eclipse.egit.ui.UIIcons;
 import org.eclipse.egit.ui.UIPreferences;
 import org.eclipse.egit.ui.UIText;
 import org.eclipse.egit.ui.UIUtils;
+import org.eclipse.egit.ui.internal.history.SWTCommitList.SWTLane;
 import org.eclipse.egit.ui.internal.history.command.HistoryViewCommands;
+import org.eclipse.egit.ui.internal.trace.GitTraceLocation;
 import org.eclipse.jface.action.Action;
 import org.eclipse.jface.action.IAction;
 import org.eclipse.jface.action.MenuManager;
 import org.eclipse.jface.action.Separator;
+import org.eclipse.jface.text.AbstractHoverInformationControlManager;
+import org.eclipse.jface.text.AbstractReusableInformationControlCreator;
+import org.eclipse.jface.text.DefaultInformationControl;
+import org.eclipse.jface.text.IInformationControl;
 import org.eclipse.jface.viewers.ColumnWeightData;
 import org.eclipse.jface.viewers.IOpenListener;
 import org.eclipse.jface.viewers.ISelection;
@@ -75,17 +82,14 @@ import org.eclipse.swt.events.FocusListener;
 import org.eclipse.swt.events.MenuDetectEvent;
 import org.eclipse.swt.events.MenuDetectListener;
 import org.eclipse.swt.events.MouseEvent;
-import org.eclipse.swt.events.MouseMoveListener;
-import org.eclipse.swt.events.MouseTrackAdapter;
-import org.eclipse.swt.graphics.Color;
 import org.eclipse.swt.graphics.Font;
 import org.eclipse.swt.graphics.FontData;
+import org.eclipse.swt.graphics.GC;
 import org.eclipse.swt.graphics.Point;
-import org.eclipse.swt.layout.FillLayout;
+import org.eclipse.swt.graphics.Rectangle;
 import org.eclipse.swt.widgets.Composite;
 import org.eclipse.swt.widgets.Control;
 import org.eclipse.swt.widgets.Event;
-import org.eclipse.swt.widgets.Label;
 import org.eclipse.swt.widgets.Listener;
 import org.eclipse.swt.widgets.Shell;
 import org.eclipse.swt.widgets.Table;
@@ -131,10 +135,10 @@ class CommitGraphTable {
 
 	private final Font hFont;
 
-	private final Color infoBackgroundColor;
-
 	private SWTCommitList allCommits;
 
+	private int allCommitsLength = 0;
+
 	// used for resolving PlotCommit objects by ids
 	private HashMap<String, PlotCommit> commitsMap = null;
 
@@ -144,25 +148,39 @@ class CommitGraphTable {
 
 	IAction copy;
 
-	private Shell hoverShell;
-
 	MenuListener menuListener;
 
 	private RevCommit commitToShow;
 
 	private GraphLabelProvider graphLabelProvider;
 
-	CommitGraphTable(Composite parent) {
+	private final TableLoader tableLoader;
+
+	private boolean trace = GitTraceLocation.HISTORYVIEW.isActive();
+
+	CommitGraphTable(Composite parent, final TableLoader loader) {
 		nFont = UIUtils.getFont(UIPreferences.THEME_CommitGraphNormalFont);
 		hFont = highlightFont();
-		infoBackgroundColor = parent.getDisplay().getSystemColor(
-				SWT.COLOR_INFO_BACKGROUND);
+		tableLoader = loader;
 
-		Table rawTable = new Table(parent, SWT.MULTI | SWT.H_SCROLL
+		final Table rawTable = new Table(parent, SWT.MULTI | SWT.H_SCROLL
 				| SWT.V_SCROLL | SWT.BORDER | SWT.FULL_SELECTION | SWT.VIRTUAL);
 		rawTable.setHeaderVisible(true);
 		rawTable.setLinesVisible(false);
 		rawTable.setFont(nFont);
+		rawTable.addListener(SWT.SetData, new Listener() {
+			public void handleEvent(Event event) {
+				if (tableLoader != null) {
+					TableItem item = (TableItem) event.item;
+					int index = rawTable.indexOf(item);
+					if (trace)
+						GitTraceLocation.getTrace().trace(
+								GitTraceLocation.HISTORYVIEW.getLocation(),
+								"Item " + index); //$NON-NLS-1$
+					tableLoader.loadItem(index);
+				}
+			}
+		});
 
 		final TableLayout layout = new TableLayout();
 		rawTable.setLayout(layout);
@@ -209,61 +227,8 @@ class CommitGraphTable {
 			}
 		});
 
-		table.getTable().addMouseTrackListener(new MouseTrackAdapter() {
-			@Override
-			public void mouseHover(MouseEvent e) {
-				synchronized (this) {
-					disposeHover();
-
-					TableItem item = table.getTable().getItem(
-							new Point(e.x, e.y));
-					if (item == null)
-						return;
-					SWTCommit commit = (SWTCommit) item.getData();
-					if (commit == null || commit.getRefCount() == 0)
-						return;
-
-					int relativeX = e.x - item.getBounds().x;
-					for (int i = 0; i < commit.getRefCount(); i++) {
-						Point textSpan = renderer.getRefHSpan(commit.getRef(i));
-						if ((textSpan != null)
-								&& (relativeX >= textSpan.x && relativeX <= textSpan.y)) {
-							hoverShell = new Shell(getTableView().getTable()
-									.getShell(), SWT.ON_TOP | SWT.NO_FOCUS
-									| SWT.TOOL);
-							hoverShell.setLayout(new FillLayout());
-							Point tableLocation = getTableView().getTable()
-									.toControl(0, 0);
-							hoverShell.setLocation(
-									-tableLocation.x + e.x,
-									-tableLocation.y + e.y
-											- renderer.getTextHeight());
-							Label label = new Label(hoverShell, SWT.NONE);
-							label.setText(getHooverText(commit.getRef(i)));
-							label.setBackground(infoBackgroundColor);
-							hoverShell.pack();
-							hoverShell.setVisible(true);
-						}
-					}
-				}
-			}
-
-			private String getHooverText(Ref r) {
-				String name = r.getName();
-				if (r.isSymbolic())
-					name += ": " + r.getLeaf().getName(); //$NON-NLS-1$
-				return name;
-			}
-
-		});
-
-		table.getTable().addMouseMoveListener(new MouseMoveListener() {
-			public void mouseMove(MouseEvent e) {
-				synchronized (this) {
-					disposeHover();
-				}
-			}
-		});
+		final RefHoverInformationControlManager hoverManager = new RefHoverInformationControlManager();
+		hoverManager.install(table.getTable());
 
 		table.getTable().addDisposeListener(new DisposeListener() {
 
@@ -272,7 +237,7 @@ class CommitGraphTable {
 					allCommits.dispose();
 				if (renderer != null)
 					renderer.dispose();
-				disposeHover();
+				hoverManager.dispose();
 			}
 		});
 
@@ -282,8 +247,8 @@ class CommitGraphTable {
 	}
 
 	CommitGraphTable(final Composite parent, final IPageSite site,
-			final MenuManager menuMgr) {
-		this(parent);
+			final MenuManager menuMgr, final TableLoader loader) {
+		this(parent, loader);
 
 		final IAction selectAll = createStandardAction(ActionFactory.SELECT_ALL);
 		getControl().addFocusListener(new FocusListener() {
@@ -306,9 +271,8 @@ class CommitGraphTable {
 
 		getTableView().addOpenListener(new IOpenListener() {
 			public void open(OpenEvent event) {
-				if (input == null || !input.isSingleFile()) {
+				if (input == null || !input.isSingleFile())
 					return;
-				}
 
 				ICommandService srv = (ICommandService) site
 						.getService(ICommandService.class);
@@ -317,7 +281,7 @@ class CommitGraphTable {
 				Command cmd = srv.getCommand(HistoryViewCommands.SHOWVERSIONS);
 				Parameterization[] parms;
 				if (Activator.getDefault().getPreferenceStore().getBoolean(
-						UIPreferences.RESOURCEHISTORY_COMPARE_MODE)) {
+						UIPreferences.RESOURCEHISTORY_COMPARE_MODE))
 					try {
 						IParameter parm = cmd
 								.getParameter(HistoryViewCommands.COMPARE_MODE_PARAM);
@@ -327,7 +291,7 @@ class CommitGraphTable {
 						Activator.handleError(e.getMessage(), e, true);
 						parms = null;
 					}
-				} else
+				else
 					parms = null;
 				ParameterizedCommand pcmd = new ParameterizedCommand(cmd, parms);
 				try {
@@ -346,13 +310,6 @@ class CommitGraphTable {
 				getTableView(), site, copy));
 	}
 
-	void disposeHover() {
-		if (hoverShell == null)
-			return;
-		hoverShell.dispose();
-		hoverShell = null;
-	}
-
 	Control getControl() {
 		return table.getControl();
 	}
@@ -363,13 +320,14 @@ class CommitGraphTable {
 	}
 
 	void selectCommit(final RevCommit c) {
-		if (c instanceof PlotCommit) {
+		if (c instanceof PlotCommit)
 			table.setSelection(new StructuredSelection(c), true);
-		} else if (commitsMap != null) {
+		else if (commitsMap != null) {
 			PlotCommit swtCommit = commitsMap.get(c.getId().name());
-			if (swtCommit != null) {
+			if (swtCommit == null && tableLoader != null)
+				tableLoader.loadCommit(c);
+			if (swtCommit != null)
 				table.setSelection(new StructuredSelection(swtCommit), true);
-			}
 		}
 	}
 
@@ -385,6 +343,10 @@ class CommitGraphTable {
 		graphLabelProvider.setRelativeDate(booleanValue);
 	}
 
+	void setShowEmailAddresses(boolean booleanValue) {
+		graphLabelProvider.setShowEmailAddresses(booleanValue);
+	}
+
 	private boolean canDoCopy() {
 		return !table.getSelection().isEmpty();
 	}
@@ -411,21 +373,28 @@ class CommitGraphTable {
 	}
 
 	void setInput(final RevFlag hFlag, final SWTCommitList list,
-			final SWTCommit[] asArray, HistoryPageInput input) {
+			final SWTCommit[] asArray, HistoryPageInput input, boolean keepPosition) {
+		int topIndex = -1;
+		if (keepPosition)
+			topIndex = table.getTable().getTopIndex();
 		setHistoryPageInput(input);
 		final SWTCommitList oldList = allCommits;
+		if (oldList != null && oldList != list)
+			oldList.dispose();
 		highlight = hFlag;
 		allCommits = list;
+		int newAllCommitsLength = allCommits.size();
 		table.setInput(asArray);
 		if (asArray != null && asArray.length > 0) {
-			if (oldList != list) {
+			if (oldList != list || allCommitsLength < newAllCommitsLength)
 				initCommitsMap();
-			}
-		} else {
+		} else
 			table.getTable().deselectAll();
-		}
+		allCommitsLength = newAllCommitsLength;
 		if (commitToShow != null)
 			selectCommit(commitToShow);
+		if (keepPosition)
+			table.getTable().setTopIndex(topIndex);
 	}
 
 	void setHistoryPageInput(HistoryPageInput input) {
@@ -446,6 +415,19 @@ class CommitGraphTable {
 	}
 
 	private void createColumns(final Table rawTable, final TableLayout layout) {
+		final TableColumn commitId = new TableColumn(rawTable, SWT.NONE);
+		commitId.setResizable(true);
+		commitId.setText(UIText.CommitGraphTable_CommitId);
+		int minWidth;
+		GC gc = new GC(rawTable.getDisplay());
+		try {
+			gc.setFont(rawTable.getFont());
+			minWidth = gc.stringExtent("0000000").x + 5; //$NON-NLS-1$
+		} finally {
+			gc.dispose();
+		}
+		layout.addColumnData(new ColumnWeightData(3, minWidth, true));
+
 		final TableColumn graph = new TableColumn(rawTable, SWT.NONE);
 		graph.setResizable(true);
 		graph.setText(UIText.CommitGraphTable_messageColumn);
@@ -460,21 +442,21 @@ class CommitGraphTable {
 
 		final TableColumn date = new TableColumn(rawTable, SWT.NONE);
 		date.setResizable(true);
-		date.setText(UIText.HistoryPage_dateColumn);
+		date.setText(UIText.HistoryPage_authorDateColumn);
 		date.setWidth(250);
 		layout.addColumnData(new ColumnWeightData(5, true));
 
-		final TableColumn commitId = new TableColumn(rawTable, SWT.NONE);
-		commitId.setResizable(true);
-		commitId.setText(UIText.CommitGraphTable_CommitId);
-		commitId.setWidth(50);
-		layout.addColumnData(new ColumnWeightData(3, true));
-
 		final TableColumn committer = new TableColumn(rawTable, SWT.NONE);
 		committer.setResizable(true);
 		committer.setText(UIText.CommitGraphTable_Committer);
 		committer.setWidth(100);
 		layout.addColumnData(new ColumnWeightData(5, true));
+
+		final TableColumn committerDate = new TableColumn(rawTable, SWT.NONE);
+		committerDate.setResizable(true);
+		committerDate.setText(UIText.CommitGraphTable_committerDataColumn);
+		committerDate.setWidth(100);
+		layout.addColumnData(new ColumnWeightData(5, true));
 	}
 
 	private void createPaintListener(final Table rawTable) {
@@ -482,7 +464,7 @@ class CommitGraphTable {
 		//
 		rawTable.addListener(SWT.EraseItem, new Listener() {
 			public void handleEvent(final Event event) {
-				if (0 <= event.index && event.index <= 4)
+				if (0 <= event.index && event.index <= 5)
 					event.detail &= ~SWT.FOREGROUND;
 			}
 		});
@@ -496,12 +478,17 @@ class CommitGraphTable {
 
 	void doPaint(final Event event) {
 		final RevCommit c = (RevCommit) ((TableItem) event.item).getData();
+		if (c instanceof SWTCommit) {
+			final SWTLane lane = ((SWTCommit) c).getLane();
+			if (lane != null && lane.color.isDisposed())
+				return;
+		}
 		if (highlight != null && c.has(highlight))
 			event.gc.setFont(hFont);
 		else
 			event.gc.setFont(nFont);
 
-		if (event.index == 0) {
+		if (event.index == 1) {
 			renderer.paint(event, input == null ? null : input.getHead());
 			return;
 		}
@@ -548,17 +535,71 @@ class CommitGraphTable {
 
 			@Override
 			public void run() {
-				if (af == ActionFactory.SELECT_ALL) {
+				if (af == ActionFactory.SELECT_ALL)
 					table.getTable().selectAll();
-				}
-				if (af == ActionFactory.COPY) {
+				if (af == ActionFactory.COPY)
 					doCopy();
-				}
 			}
 		};
 		return action;
 	}
 
+	private static final class RefHoverInformationControlCreator extends
+			AbstractReusableInformationControlCreator {
+		@Override
+		protected IInformationControl doCreateInformationControl(Shell parent) {
+			return new DefaultInformationControl(parent);
+		}
+	}
+
+	private final class RefHoverInformationControlManager extends
+			AbstractHoverInformationControlManager {
+
+		protected RefHoverInformationControlManager() {
+			super(new RefHoverInformationControlCreator());
+		}
+
+		@Override
+		protected void computeInformation() {
+			MouseEvent e = getHoverEvent();
+
+			TableItem item = table.getTable().getItem(new Point(e.x, e.y));
+			if (item != null) {
+				SWTCommit commit = (SWTCommit) item.getData();
+				if (commit != null && commit.getRefCount() > 0) {
+					Rectangle itemBounds = item.getBounds();
+					int firstColumnWidth = table.getTable().getColumn(0).getWidth();
+					int relativeX = e.x - firstColumnWidth - itemBounds.x;
+					for (int i = 0; i < commit.getRefCount(); i++) {
+						Point textSpan = renderer.getRefHSpan(commit.getRef(i));
+						if ((textSpan != null)
+								&& (relativeX >= textSpan.x && relativeX <= textSpan.y)) {
+
+							String hoverText = getHoverText(commit.getRef(i));
+							int width = textSpan.y - textSpan.x;
+							Rectangle rectangle = new Rectangle(
+									firstColumnWidth + itemBounds.x
+											+ textSpan.x, itemBounds.y, width,
+									itemBounds.height);
+							setInformation(hoverText, rectangle);
+							return;
+						}
+					}
+				}
+			}
+
+			// computeInformation must setInformation in all cases
+			setInformation(null, null);
+		}
+
+		private String getHoverText(Ref r) {
+			String name = r.getName();
+			if (r.isSymbolic())
+				name += ": " + r.getLeaf().getName(); //$NON-NLS-1$
+			return name;
+		}
+	}
+
 	private final class CommitDragSourceListener extends DragSourceAdapter {
 		@Override
 		public void dragStart(DragSourceEvent event) {
@@ -588,9 +629,8 @@ class CommitGraphTable {
 								UIText.CommitGraphTable_UnableToWritePatch,
 								commit.getId().name()), e);
 					} finally {
-						if (patchFile != null) {
+						if (patchFile != null)
 							patchFile.deleteOnExit();
-						}
 					}
 				}
 			}
@@ -700,6 +740,10 @@ class CommitGraphTable {
 							HistoryViewCommands.OPEN_IN_TEXT_EDITOR,
 							UIText.GitHistoryPage_OpenInTextEditorLabel));
 				}
+				if (selectionSize == 1)
+					popupMgr.add(getCommandContributionItem(
+							HistoryViewCommands.SHOW_BLAME,
+							UIText.CommitFileDiffViewer_ShowAnnotationsMenuLabel));
 			}
 
 			if (selectionSize == 1) {
@@ -707,6 +751,9 @@ class CommitGraphTable {
 				popupMgr.add(getCommandContributionItem(
 						HistoryViewCommands.CHECKOUT,
 						UIText.GitHistoryPage_CheckoutMenuLabel));
+				popupMgr.add(getCommandContributionItem(
+						HistoryViewCommands.PUSH_COMMIT,
+						UIText.GitHistoryPage_pushCommit));
 				popupMgr.add(new Separator());
 				popupMgr.add(getCommandContributionItem(
 						HistoryViewCommands.CREATE_BRANCH,
@@ -714,6 +761,9 @@ class CommitGraphTable {
 				popupMgr.add(getCommandContributionItem(
 						HistoryViewCommands.DELETE_BRANCH,
 						UIText.CommitGraphTable_DeleteBranchAction));
+				popupMgr.add(getCommandContributionItem(
+						HistoryViewCommands.RENAME_BRANCH,
+						UIText.CommitGraphTable_RenameBranchMenuLabel));
 				popupMgr.add(getCommandContributionItem(
 						HistoryViewCommands.CREATE_TAG,
 						UIText.GitHistoryPage_CreateTagMenuLabel));
