@@ -22,7 +22,7 @@ import java.text.DateFormat;
 import java.text.SimpleDateFormat;
 import java.util.ArrayList;
 import java.util.Arrays;
-import java.util.HashMap;
+import java.util.Collection;
 import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
@@ -91,16 +91,20 @@ public class CommitInfoBuilder {
 
 	private Color linesRemovedColor;
 
+	private final Collection<Ref> allRefs;
+
 	/**
 	 * @param db the repository
 	 * @param commit the commit the info should be shown for
 	 * @param currentDiffs list of current diffs
 	 * @param fill whether to fill the available space
+	 * @param allRefs all Ref's to examine regarding marge bases
 	 */
-	public CommitInfoBuilder(Repository db, PlotCommit commit, List<FileDiff> currentDiffs, boolean fill) {
+	public CommitInfoBuilder(Repository db, PlotCommit commit, List<FileDiff> currentDiffs, boolean fill, Collection<Ref> allRefs) {
 		this.db = db;
 		this.commit = commit;
 		this.fill = fill;
+		this.allRefs = allRefs;
 		this.currentDiffs = new ArrayList<FileDiff>(currentDiffs);
 	}
 
@@ -167,7 +171,8 @@ public class CommitInfoBuilder {
 		}
 
 		for (int i = 0; i < commit.getParentCount(); i++) {
-			final RevCommit p = commit.getParent(i);
+			final SWTCommit p = (SWTCommit)commit.getParent(i);
+			p.parseBody();
 			d.append(UIText.CommitMessageViewer_parent);
 			d.append(": "); //$NON-NLS-1$
 			addLink(d, styles, p);
@@ -178,7 +183,8 @@ public class CommitInfoBuilder {
 		}
 
 		for (int i = 0; i < commit.getChildCount(); i++) {
-			final RevCommit p = commit.getChild(i);
+			final SWTCommit p = (SWTCommit)commit.getChild(i);
+			p.parseBody();
 			d.append(UIText.CommitMessageViewer_child);
 			d.append(": "); //$NON-NLS-1$
 			addLink(d, styles, p);
@@ -320,14 +326,10 @@ public class CommitInfoBuilder {
 			// search-for commit is found. This is quite likely, so optimize for this.
 			revWalk.markStart(Arrays.asList(commit.getParents()));
 			ObjectIdSubclassMap<ObjectId> cutOff = new ObjectIdSubclassMap<ObjectId>();
-			Map<String, Ref> refsMap = new HashMap<String, Ref>();
-			refsMap.putAll(db.getRefDatabase().getRefs(Constants.R_HEADS));
-			// add remote heads to search
-			refsMap.putAll(db.getRefDatabase().getRefs(Constants.R_REMOTES));
 
 			final int SKEW = 24*3600; // one day clock skew
 
-			for (Ref ref : refsMap.values()) {
+			for (Ref ref : allRefs) {
 				RevCommit headCommit = revWalk.parseCommit(ref.getObjectId());
 
 				// if commit is in the ref branch, then the tip of ref should be
