@@ -3,7 +3,7 @@
  * Copyright (C) 2008, Shawn O. Pearce <spearce@spearce.org>
  * Copyright (c) 2010, Stefan Lay <stefan.lay@sap.com>
  * Copyright (C) 2010, Mathias Kinzler <mathias.kinzler@sap.com>
- * Copyright (C) 2010-2011, Matthias Sohn <matthias.sohn@sap.com>
+ * Copyright (C) 2010-2012, Matthias Sohn <matthias.sohn@sap.com>
  * Copyright (C) 2012, Daniel megert <daniel_megert@ch.ibm.com>
  *
  * All rights reserved. This program and the accompanying materials
@@ -24,9 +24,8 @@ import org.eclipse.core.resources.IWorkspaceRoot;
 import org.eclipse.core.runtime.IAdaptable;
 import org.eclipse.core.runtime.IPath;
 import org.eclipse.core.runtime.Path;
-import org.eclipse.core.runtime.jobs.IJobChangeEvent;
+import org.eclipse.core.runtime.jobs.ISchedulingRule;
 import org.eclipse.core.runtime.jobs.Job;
-import org.eclipse.core.runtime.jobs.JobChangeAdapter;
 import org.eclipse.egit.core.AdapterUtils;
 import org.eclipse.egit.core.project.RepositoryMapping;
 import org.eclipse.egit.ui.Activator;
@@ -113,7 +112,10 @@ import org.eclipse.ui.dialogs.PreferencesUtil;
 import org.eclipse.ui.progress.IWorkbenchSiteProgressService;
 
 /** Graphical commit history viewer. */
-public class GitHistoryPage extends HistoryPage implements RefsChangedListener {
+public class GitHistoryPage extends HistoryPage implements RefsChangedListener,
+		ISchedulingRule, TableLoader {
+
+	private static final int INITIAL_ITEM = -1;
 
 	/** actions used in GitHistoryPage **/
 	private static class GitHistoryPageActions {
@@ -131,13 +133,12 @@ public class GitHistoryPage extends HistoryPage implements RefsChangedListener {
 
 			public void run() {
 				historyPage.store.setValue(prefName, isChecked());
-				if (historyPage.store.needsSaving()) {
+				if (historyPage.store.needsSaving())
 					try {
 						historyPage.store.save();
 					} catch (IOException e) {
 						Activator.handleError(e.getMessage(), e, false);
 					}
-				}
 			}
 
 			abstract void apply(boolean value);
@@ -171,13 +172,12 @@ public class GitHistoryPage extends HistoryPage implements RefsChangedListener {
 			public void run() {
 				String oldName = historyPage.getName();
 				String oldDescription = historyPage.getDescription();
-				if (!isChecked()) {
+				if (!isChecked())
 					if (historyPage.showAllFilter == filter) {
 						historyPage.showAllFilter = ShowFilter.SHOWALLRESOURCE;
 						showAllResourceVersionsAction.setChecked(true);
 						historyPage.initAndStartRevWalk(false);
 					}
-				}
 				if (isChecked() && historyPage.showAllFilter != filter) {
 					historyPage.showAllFilter = filter;
 					if (this != showAllRepoVersionsAction)
@@ -212,6 +212,8 @@ public class GitHistoryPage extends HistoryPage implements RefsChangedListener {
 
 		BooleanPrefAction showRelativeDateAction;
 
+		BooleanPrefAction showEmailAddressesAction;
+
 		BooleanPrefAction showNotesAction;
 
 		BooleanPrefAction wrapCommentAction;
@@ -230,6 +232,8 @@ public class GitHistoryPage extends HistoryPage implements RefsChangedListener {
 
 		IWorkbenchAction showAllBranchesAction;
 
+		IWorkbenchAction showAdditionalRefsAction;
+
 		BooleanPrefAction followRenamesAction;
 
 		IWorkbenchAction reuseCompareEditorAction;
@@ -257,9 +261,11 @@ public class GitHistoryPage extends HistoryPage implements RefsChangedListener {
 			createCompareModeAction();
 			createReuseCompareEditorAction();
 			createShowAllBranchesAction();
+			createShowAdditionalRefsAction();
 			createShowCommentAction();
 			createShowFilesAction();
 			createShowRelativeDateAction();
+			createShowEmailAddressesAction();
 			createShowNotesAction();
 			createWrapCommentAction();
 			createFillCommentAction();
@@ -276,13 +282,12 @@ public class GitHistoryPage extends HistoryPage implements RefsChangedListener {
 					historyPage.store.setValue(
 							UIPreferences.RESOURCEHISTORY_SHOW_FINDTOOLBAR,
 							isChecked());
-					if (historyPage.store.needsSaving()) {
+					if (historyPage.store.needsSaving())
 						try {
 							historyPage.store.save();
 						} catch (IOException e) {
 							Activator.handleError(e.getMessage(), e, false);
 						}
-					}
 					historyPage.layout();
 				}
 			};
@@ -368,6 +373,19 @@ public class GitHistoryPage extends HistoryPage implements RefsChangedListener {
 			actionsToDispose.add(showAllBranchesAction);
 		}
 
+		private void createShowAdditionalRefsAction() {
+			showAdditionalRefsAction = new BooleanPrefAction(
+					UIPreferences.RESOURCEHISTORY_SHOW_ADDITIONAL_REFS,
+					UIText.GitHistoryPage_ShowAdditionalRefsMenuLabel) {
+
+				@Override
+				void apply(boolean value) {
+					historyPage.refresh();
+				}
+			};
+			actionsToDispose.add(showAdditionalRefsAction);
+		}
+
 		private void createFollowRenamesAction() {
 			followRenamesAction = new BooleanPrefAction(
 					UIPreferences.RESOURCEHISTORY_FOLLOW_RENAMES,
@@ -417,6 +435,18 @@ public class GitHistoryPage extends HistoryPage implements RefsChangedListener {
 			actionsToDispose.add(showRelativeDateAction);
 		}
 
+		private void createShowEmailAddressesAction() {
+			showEmailAddressesAction = new BooleanPrefAction(
+					UIPreferences.RESOURCEHISTORY_SHOW_EMAIL_ADDRESSES,
+					UIText.GitHistoryPage_toggleEmailAddresses) {
+				void apply(boolean date) {
+					// nothing, just set the Preference
+				}
+			};
+			showEmailAddressesAction.apply(showEmailAddressesAction.isChecked());
+			actionsToDispose.add(showEmailAddressesAction);
+		}
+
 		private void createShowNotesAction() {
 			showNotesAction = new BooleanPrefAction(
 					UIPreferences.RESOURCEHISTORY_SHOW_NOTES,
@@ -579,9 +609,6 @@ public class GitHistoryPage extends HistoryPage implements RefsChangedListener {
 	/** Job that is updating our history view, if we are refreshing. */
 	private GenerateHistoryJob job;
 
-	/** Revision walker that allocated our graph's commit nodes. */
-	private SWTWalk currentWalk;
-
 	/** Last HEAD */
 	private AnyObjectId currentHeadId;
 
@@ -590,6 +617,8 @@ public class GitHistoryPage extends HistoryPage implements RefsChangedListener {
 
 	private boolean currentShowAllBranches;
 
+	private boolean currentShowAdditionalRefs;
+
 	private boolean currentShowNotes;
 
 	private boolean currentFollowRenames;
@@ -605,25 +634,24 @@ public class GitHistoryPage extends HistoryPage implements RefsChangedListener {
 					graph.getTableView().refresh();
 				}
 			}
+			if (UIPreferences.RESOURCEHISTORY_SHOW_EMAIL_ADDRESSES.equals(prop)) {
+				Object oldValue = event.getOldValue();
+				if (oldValue == null || !oldValue.equals(event.getNewValue())) {
+					graph.setShowEmailAddresses(isShowingEmailAddresses());
+					graph.getTableView().refresh();
+				}
+			}
 			if (UIPreferences.HISTORY_MAX_BRANCH_LENGTH.equals(prop)
 					|| UIPreferences.HISTORY_MAX_TAG_LENGTH.equals(prop))
 				graph.getTableView().refresh();
 		}
 	};
 
-	/**
-	 * Highlight flag that can be applied to commits to make them stand out.
-	 * <p>
-	 * Allocated at the same time as {@link #currentWalk}. If the walk rebuilds,
-	 * so must this flag.
-	 */
-	private RevFlag highlightFlag;
 
 	/**
-	 * List of paths we used to limit {@link #currentWalk}; null if no paths.
+	 * List of paths we used to limit the revwalk; null if no paths.
 	 * <p>
-	 * Note that a change in this list requires that {@link #currentWalk} and
-	 * all of its associated commits.
+	 * Note that a change in this list requires that the history is redrawn
 	 */
 	private List<FilterPath> pathFilters;
 
@@ -639,13 +667,11 @@ public class GitHistoryPage extends HistoryPage implements RefsChangedListener {
 	 *         FOLDER or PROJECT and we can show it; false otherwise.
 	 */
 	public static boolean canShowHistoryFor(final Object object) {
-		if (object instanceof HistoryPageInput) {
+		if (object instanceof HistoryPageInput)
 			return true;
-		}
 
-		if (object instanceof IResource) {
+		if (object instanceof IResource)
 			return typeOk((IResource) object);
-		}
 
 		if (object instanceof RepositoryTreeNode)
 			return true;
@@ -701,7 +727,7 @@ public class GitHistoryPage extends HistoryPage implements RefsChangedListener {
 		preferencesLink.addSelectionListener(new SelectionAdapter() {
 			@Override
 			public void widgetSelected(SelectionEvent e) {
-				String preferencePageId = "org.eclipse.egit.ui.GitPreferences"; //$NON-NLS-1$
+				String preferencePageId = "org.eclipse.egit.ui.internal.preferences.HistoryPreferencePage"; //$NON-NLS-1$
 				PreferenceDialog dialog = PreferencesUtil
 						.createPreferenceDialogOn(getSite().getShell(), preferencePageId,
 								new String[] { preferencePageId }, null);
@@ -713,9 +739,10 @@ public class GitHistoryPage extends HistoryPage implements RefsChangedListener {
 		graphDetailSplit = new SashForm(historyControl, SWT.VERTICAL);
 		GridDataFactory.fillDefaults().grab(true, true).applyTo(
 				graphDetailSplit);
-		graph = new CommitGraphTable(graphDetailSplit, getSite(), popupMgr);
+		graph = new CommitGraphTable(graphDetailSplit, getSite(), popupMgr, this);
 
 		graph.setRelativeDate(isShowingRelativeDates());
+		graph.setShowEmailAddresses(isShowingEmailAddresses());
 		Activator.getDefault().getPreferenceStore()
 				.addPropertyChangeListener(listener);
 
@@ -773,7 +800,6 @@ public class GitHistoryPage extends HistoryPage implements RefsChangedListener {
 		// use the same font as in message viewer
 		errorText.setFont(UIUtils
 				.getFont(UIPreferences.THEME_CommitMessageFont));
-		errorText.setText(UIText.CommitFileDiffViewer_SelectOneCommitMessage);
 
 		final GridLayout parentLayout = new GridLayout();
 		parentLayout.marginHeight = 0;
@@ -799,15 +825,12 @@ public class GitHistoryPage extends HistoryPage implements RefsChangedListener {
 			revInfoSplit.setMaximizedControl(commentViewer.getControl());
 		} else if (!showComment && showFiles) {
 			graphDetailSplit.setMaximizedControl(null);
-			// the parent of the control!
-			revInfoSplit.setMaximizedControl(fileViewer.getControl()
-					.getParent());
-		} else if (!showComment && !showFiles) {
+			revInfoSplit.setMaximizedControl(fileViewer.getControl());
+		} else if (!showComment && !showFiles)
 			graphDetailSplit.setMaximizedControl(graph.getControl());
-		}
-		if (showFindToolbar) {
+		if (showFindToolbar)
 			((GridData) findToolbar.getLayoutData()).heightHint = SWT.DEFAULT;
-		} else {
+		else {
 			((GridData) findToolbar.getLayoutData()).heightHint = 0;
 			findToolbar.clear();
 		}
@@ -901,12 +924,16 @@ public class GitHistoryPage extends HistoryPage implements RefsChangedListener {
 				UIText.GitHistoryPage_ShowSubMenuLabel);
 		viewMenuMgr.add(showSubMenuMgr);
 		showSubMenuMgr.add(actions.showAllBranchesAction);
+		showSubMenuMgr.add(actions.showAdditionalRefsAction);
 		showSubMenuMgr.add(actions.showNotesAction);
 		showSubMenuMgr.add(actions.followRenamesAction);
 		showSubMenuMgr.add(new Separator());
 		showSubMenuMgr.add(actions.findAction);
 		showSubMenuMgr.add(actions.showFilesAction);
 		showSubMenuMgr.add(actions.showCommentAction);
+		showSubMenuMgr.add(new Separator());
+		showSubMenuMgr.add(actions.showRelativeDateAction);
+		showSubMenuMgr.add(actions.showEmailAddressesAction);
 
 		IMenuManager filterSubMenuMgr = new MenuManager(
 				UIText.GitHistoryPage_FilterSubMenuLabel);
@@ -923,9 +950,6 @@ public class GitHistoryPage extends HistoryPage implements RefsChangedListener {
 		viewMenuMgr.add(new Separator());
 		viewMenuMgr.add(actions.wrapCommentAction);
 		viewMenuMgr.add(actions.fillCommentAction);
-
-		viewMenuMgr.add(new Separator());
-		viewMenuMgr.add(actions.showRelativeDateAction);
 	}
 
 	public void dispose() {
@@ -946,17 +970,15 @@ public class GitHistoryPage extends HistoryPage implements RefsChangedListener {
 		for (IWorkbenchAction action : actions.actionsToDispose)
 			action.dispose();
 		actions.actionsToDispose.clear();
-		cancelRefreshJob();
+		releaseGenerateHistoryJob();
 		if (popupMgr != null) {
-			for (final IContributionItem i : popupMgr.getItems()) {
+			for (final IContributionItem i : popupMgr.getItems())
 				if (i instanceof IWorkbenchAction)
 					((IWorkbenchAction) i).dispose();
-			}
 			for (final IContributionItem i : getSite().getActionBars()
-					.getMenuManager().getItems()) {
+					.getMenuManager().getItems())
 				if (i instanceof IWorkbenchAction)
 					((IWorkbenchAction) i).dispose();
-			}
 		}
 		super.dispose();
 	}
@@ -1080,7 +1102,6 @@ public class GitHistoryPage extends HistoryPage implements RefsChangedListener {
 			if (this.input != null)
 				return true;
 
-			cancelRefreshJob();
 			Object o = super.getInput();
 			if (o == null) {
 				setErrorMessage(UIText.GitHistoryPage_NoInputMessage);
@@ -1161,20 +1182,9 @@ public class GitHistoryPage extends HistoryPage implements RefsChangedListener {
 				setErrorMessage(UIText.GitHistoryPage_NoInputMessage);
 				return false;
 			}
-			Repository db = input.getRepository();
-			if (resolveHead(db, true) == null) {
-				this.name = ""; //$NON-NLS-1$
-				setErrorMessage(UIText.GitHistoryPage_NoInputMessage);
-				return false;
-			}
 
 			final IResource[] inResources = input.getItems();
 			final File[] inFiles = input.getFileList();
-			if (inResources != null && inResources.length == 0) {
-				this.name = ""; //$NON-NLS-1$
-				setErrorMessage(UIText.GitHistoryPage_NoInputMessage);
-				return false;
-			}
 
 			this.name = calculateName(input);
 
@@ -1239,9 +1249,9 @@ public class GitHistoryPage extends HistoryPage implements RefsChangedListener {
 		try {
 			RevCommit c = null;
 			RevObject any = rw.parseAny(ref.getLeaf().getObjectId());
-			if (any instanceof RevCommit) {
+			if (any instanceof RevCommit)
 				c = (RevCommit) any;
-			} else if (any instanceof RevTag) {
+			else if (any instanceof RevTag) {
 				RevTag t = rw.parseTag(any);
 				Object anyCommit = rw.parseAny(t.getObject());
 				if (anyCommit instanceof RevCommit)
@@ -1263,11 +1273,11 @@ public class GitHistoryPage extends HistoryPage implements RefsChangedListener {
 		// as it wrongly pollutes the navigation history
 		final String repositoryName = Activator.getDefault()
 				.getRepositoryUtil().getRepositoryName(in.getRepository());
-		if (in.getItems() == null && in.getFileList() == null) {
+		if (in.getItems() == null && in.getFileList() == null)
 			// plain repository, no files specified
 			return NLS.bind(UIText.GitHistoryPage_RepositoryNamePattern,
 					repositoryName);
-		} else if (in.getItems() != null && in.getItems().length == 1) {
+		else if (in.getItems() != null && in.getItems().length == 1) {
 			// single resource
 			IResource resource = in.getItems()[0];
 			final String type;
@@ -1420,19 +1430,22 @@ public class GitHistoryPage extends HistoryPage implements RefsChangedListener {
 		return this.input;
 	}
 
+	@SuppressWarnings("boxing")
 	void showCommitList(final Job j, final SWTCommitList list,
-			final SWTCommit[] asArray, final boolean incomplete) {
+			final SWTCommit[] asArray, final RevCommit toSelect, final boolean incomplete, final RevFlag highlightFlag) {
 		if (trace)
 			GitTraceLocation.getTrace().traceEntry(
 					GitTraceLocation.HISTORYVIEW.getLocation(),
-					new Object[] { list, asArray });
+					new Object[] { list.size()});
 		if (job != j || graph.getControl().isDisposed())
 			return;
 
 		graph.getControl().getDisplay().asyncExec(new Runnable() {
 			public void run() {
 				if (!graph.getControl().isDisposed() && job == j) {
-					graph.setInput(highlightFlag, list, asArray, input);
+					graph.setInput(highlightFlag, list, asArray, input, true);
+					if (toSelect != null)
+						graph.selectCommit(toSelect);
 					if (trace)
 						GitTraceLocation.getTrace().trace(
 								GitTraceLocation.HISTORYVIEW.getLocation(),
@@ -1471,8 +1484,6 @@ public class GitHistoryPage extends HistoryPage implements RefsChangedListener {
 				GitTraceLocation.getTrace().traceEntry(
 						GitTraceLocation.HISTORYVIEW.getLocation());
 
-			cancelRefreshJob();
-
 			if (input == null)
 				return;
 			Repository db = input.getRepository();
@@ -1482,25 +1493,25 @@ public class GitHistoryPage extends HistoryPage implements RefsChangedListener {
 			}
 
 			AnyObjectId headId = resolveHead(db, true);
-			if (headId == null)
+			if (headId == null) {
+				graph.getTableView().setInput(new SWTCommit[0]);
+				currentHeadId = null;
 				return;
+			}
 
 			List<FilterPath> paths = buildFilterPaths(input.getItems(), input
 					.getFileList(), db);
 
 			if (forceNewWalk || shouldRedraw(db, headId, paths)) {
-				// TODO Do not dispose SWTWalk just because HEAD changed
-				// In theory we should be able to update the graph and
-				// not dispose of the SWTWalk, even if HEAD was reset to
-				// HEAD^1 and the old HEAD commit should not be visible.
-				//
-				createNewWalk(db, headId);
-				setWalkStartPoints(db, headId);
+				releaseGenerateHistoryJob();
 
-				setupFileViewer(db, paths);
+				SWTWalk walk = createNewWalk(db, headId);
+				setWalkStartPoints(walk, db, headId);
+
+				setupFileViewer(walk, db, paths);
 				setupCommentViewer(db);
 
-				scheduleNewGenerateHistoryJob();
+				loadHistory(INITIAL_ITEM, walk);
 			} else
 				// needed for context menu and double click
 				graph.setHistoryPageInput(input);
@@ -1514,7 +1525,7 @@ public class GitHistoryPage extends HistoryPage implements RefsChangedListener {
 
 	private boolean shouldRedraw(Repository db, AnyObjectId headId, List<FilterPath> paths) {
 		boolean pathChanged = pathChanged(pathFilters, paths);
-		boolean headChanged = !headId.equals(currentHeadId);
+		boolean headChanged = headId == null || !headId.equals(currentHeadId);
 		boolean repoChanged = false;
 
 		boolean allBranchesChanged = currentShowAllBranches != store
@@ -1522,6 +1533,11 @@ public class GitHistoryPage extends HistoryPage implements RefsChangedListener {
 		currentShowAllBranches = store
 			.getBoolean(UIPreferences.RESOURCEHISTORY_SHOW_ALL_BRANCHES);
 
+		boolean additionalRefsChange = currentShowAdditionalRefs != store
+				.getBoolean(UIPreferences.RESOURCEHISTORY_SHOW_ADDITIONAL_REFS);
+		currentShowAdditionalRefs = store
+				.getBoolean(UIPreferences.RESOURCEHISTORY_SHOW_ADDITIONAL_REFS);
+
 		boolean showNotesChanged = currentShowNotes != store
 				.getBoolean(UIPreferences.RESOURCEHISTORY_SHOW_NOTES);
 		currentShowNotes = store
@@ -1535,8 +1551,8 @@ public class GitHistoryPage extends HistoryPage implements RefsChangedListener {
 		}
 
 		return pathChanged
-			|| currentWalk == null || headChanged || repoChanged || allBranchesChanged
-			|| showNotesChanged || followRenamesChanged;
+			|| headChanged || repoChanged || allBranchesChanged
+			|| additionalRefsChange || showNotesChanged || followRenamesChanged;
 	}
 
 	/**
@@ -1574,10 +1590,9 @@ public class GitHistoryPage extends HistoryPage implements RefsChangedListener {
 				final RepositoryMapping map = RepositoryMapping.getMapping(r);
 				if (map == null)
 					continue;
-				if (db != map.getRepository()) {
+				if (db != map.getRepository())
 					throw new IllegalStateException(
 							UIText.AbstractHistoryCommanndHandler_NoUniqueRepository);
-				}
 
 				if (showAllFilter == ShowFilter.SHOWALLFOLDER) {
 					final String path;
@@ -1614,31 +1629,28 @@ public class GitHistoryPage extends HistoryPage implements RefsChangedListener {
 					filePath = new Path(file.getParentFile().getPath());
 					isRegularFile = false;
 				} else if (showAllFilter == ShowFilter.SHOWALLPROJECT
-						|| showAllFilter == ShowFilter.SHOWALLREPO) {
+						|| showAllFilter == ShowFilter.SHOWALLREPO)
 					// we don't know of projects here -> treat as SHOWALLREPO
 					continue;
-				} else /* if (showAllFilter == ShowFilter.SHOWALLRESOURCE) */{
+				else /* if (showAllFilter == ShowFilter.SHOWALLRESOURCE) */{
 					filePath = new Path(file.getPath());
 					isRegularFile = file.isFile();
 				}
 
-				if (gitDirPath.isPrefixOf(filePath)) {
+				if (gitDirPath.isPrefixOf(filePath))
 					throw new IllegalStateException(
 							NLS
 									.bind(
 											UIText.GitHistoryPage_FileOrFolderPartOfGitDirMessage,
 											filePath.toOSString()));
-				}
 
 				IPath pathToAdd = filePath.removeFirstSegments(segmentCount)
 						.setDevice(null);
-				if (!pathToAdd.isEmpty()) {
+				if (!pathToAdd.isEmpty())
 					paths.add(new FilterPath(pathToAdd.toString(), isRegularFile));
-				}
 			}
-		} else {
+		} else
 			paths = new ArrayList<FilterPath>(0);
-		}
 		return paths;
 	}
 
@@ -1648,43 +1660,46 @@ public class GitHistoryPage extends HistoryPage implements RefsChangedListener {
 		return !o.equals(n);
 	}
 
-	private void createNewWalk(Repository db, AnyObjectId headId) {
+	private SWTWalk createNewWalk(Repository db, AnyObjectId headId) {
 		currentHeadId = headId;
-		if (currentWalk != null)
-			currentWalk.release();
-		currentWalk = new SWTWalk(db);
+		SWTWalk walk = new SWTWalk(db);
 		try {
-			currentWalk.addAdditionalRefs(db.getRefDatabase().getAdditionalRefs());
-			currentWalk.addAdditionalRefs(db.getRefDatabase().
-					getRefs(Constants.R_NOTES).values());
+			if (store
+					.getBoolean(UIPreferences.RESOURCEHISTORY_SHOW_ADDITIONAL_REFS))
+				walk.addAdditionalRefs(db.getRefDatabase()
+						.getAdditionalRefs());
+			walk.addAdditionalRefs(db.getRefDatabase()
+					.getRefs(Constants.R_NOTES).values());
 		} catch (IOException e) {
 			throw new IllegalStateException(NLS.bind(
 					UIText.GitHistoryPage_errorReadingAdditionalRefs, Activator
 							.getDefault().getRepositoryUtil()
 							.getRepositoryName(db)), e);
 		}
-		currentWalk.sort(RevSort.COMMIT_TIME_DESC, true);
-		currentWalk.sort(RevSort.BOUNDARY, true);
-		highlightFlag = currentWalk.newFlag("highlight"); //$NON-NLS-1$
+		walk.sort(RevSort.COMMIT_TIME_DESC, true);
+		walk.sort(RevSort.BOUNDARY, true);
+		walk.setRetainBody(false);
+		return walk;
 	}
 
-	private void setWalkStartPoints(Repository db, AnyObjectId headId) {
+	private void setWalkStartPoints(RevWalk walk, Repository db, AnyObjectId headId) {
 		try {
 			if (store
 					.getBoolean(UIPreferences.RESOURCEHISTORY_SHOW_ALL_BRANCHES)) {
-				markStartAllRefs(Constants.R_HEADS);
-				markStartAllRefs(Constants.R_REMOTES);
-				markStartAllRefs(Constants.R_TAGS);
-				markStartAdditionalRefs();
+				markStartAllRefs(walk, Constants.R_HEADS);
+				markStartAllRefs(walk, Constants.R_REMOTES);
+				markStartAllRefs(walk, Constants.R_TAGS);
 			}
+			if (store
+					.getBoolean(UIPreferences.RESOURCEHISTORY_SHOW_ADDITIONAL_REFS))
+				markStartAdditionalRefs(walk);
 			if (store
 					.getBoolean(UIPreferences.RESOURCEHISTORY_SHOW_NOTES))
-				markStartAllRefs(Constants.R_NOTES);
-			else {
-				markUninteresting(Constants.R_NOTES);
-			}
+				markStartAllRefs(walk, Constants.R_NOTES);
+			else
+				markUninteresting(walk, Constants.R_NOTES);
 
-			currentWalk.markStart(currentWalk.parseCommit(headId));
+			walk.markStart(walk.parseCommit(headId));
 		} catch (IOException e) {
 			throw new IllegalStateException(NLS.bind(
 					UIText.GitHistoryPage_errorSettingStartPoints, Activator
@@ -1698,15 +1713,15 @@ public class GitHistoryPage extends HistoryPage implements RefsChangedListener {
 		commentViewer.refresh();
 	}
 
-	private TreeWalk setupFileViewer(Repository db, List<FilterPath> paths) {
-		final TreeWalk fileWalker = createFileWalker(db, paths);
+	private TreeWalk setupFileViewer(RevWalk walk, Repository db, List<FilterPath> paths) {
+		final TreeWalk fileWalker = createFileWalker(walk, db, paths);
 		fileViewer.setTreeWalk(db, fileWalker);
 		fileViewer.refresh();
 		fileViewer.addSelectionChangedListener(commentViewer);
 		return fileWalker;
 	}
 
-	private TreeWalk createFileWalker(Repository db, List<FilterPath> paths) {
+	private TreeWalk createFileWalker(RevWalk walk, Repository db, List<FilterPath> paths) {
 		final TreeWalk fileWalker = new TreeWalk(db);
 		fileWalker.setRecursive(true);
 		fileWalker.setFilter(TreeFilter.ANY_DIFF);
@@ -1720,18 +1735,18 @@ public class GitHistoryPage extends HistoryPage implements RefsChangedListener {
 				selectedPaths.add(filterPath.getPath());
 
 			TreeFilter followFilter = createFollowFilterFor(selectedPaths);
-			currentWalk.setTreeFilter(followFilter);
+			walk.setTreeFilter(followFilter);
 		} else if (paths.size() > 0) {
 			pathFilters = paths;
 			List<String> stringPaths = new ArrayList<String>(paths.size());
 			for (FilterPath p : paths)
 				stringPaths.add(p.getPath());
 
-			currentWalk.setTreeFilter(AndTreeFilter.create(PathFilterGroup
+			walk.setTreeFilter(AndTreeFilter.create(PathFilterGroup
 					.createFromStrings(stringPaths), TreeFilter.ANY_DIFF));
 		} else {
 			pathFilters = null;
-			currentWalk.setTreeFilter(TreeFilter.ALL);
+			walk.setTreeFilter(TreeFilter.ALL);
 		}
 		return fileWalker;
 	}
@@ -1762,38 +1777,44 @@ public class GitHistoryPage extends HistoryPage implements RefsChangedListener {
 	 * @param paths the paths to check
 	 */
 	private boolean allRegularFiles(List<FilterPath> paths) {
-		for (FilterPath filterPath : paths) {
+		for (FilterPath filterPath : paths)
 			if (!filterPath.isRegularFile())
 				return false;
-		}
 		return true;
 	}
 
-	private void scheduleNewGenerateHistoryJob() {
-		final SWTCommitList list = new SWTCommitList(graph.getControl()
-				.getDisplay());
-		list.source(currentWalk);
-		final GenerateHistoryJob rj = new GenerateHistoryJob(this, list);
-		rj.addJobChangeListener(new JobChangeAdapter() {
-			@Override
-			public void done(final IJobChangeEvent event) {
-				final Control graphctl = graph.getControl();
-				if (job != rj || graphctl.isDisposed())
-					return;
-				graphctl.getDisplay().asyncExec(new Runnable() {
-					public void run() {
-						if (job == rj)
-							job = null;
-					}
-				});
-			}
-		});
-		job = rj;
+	public void loadItem(int item) {
+		if (job == null || job.loadMoreItemsThreshold() < item)
+			loadHistory(item, null);
+	}
+
+	public void loadCommit(RevCommit c) {
+		if (job == null)
+			return;
+		job.setLoadHint(c);
 		if (trace)
 			GitTraceLocation.getTrace().trace(
 					GitTraceLocation.HISTORYVIEW.getLocation(),
 					"Scheduling GenerateHistoryJob"); //$NON-NLS-1$
-		schedule(rj);
+		schedule(job);
+	}
+
+	/**
+	 * Load history items incrementally
+	 * @param itemToLoad hint for index of item that should be loaded
+	 * @param walk the revwalk, used only if itemToLoad ==  INITIAL_ITEM
+	 */
+	private void loadHistory(final int itemToLoad, RevWalk walk) {
+		if (itemToLoad == INITIAL_ITEM) {
+			job = new GenerateHistoryJob(this, graph.getControl(), walk);
+			job.setRule(this);
+		}
+		job.setLoadHint(itemToLoad);
+		if (trace)
+			GitTraceLocation.getTrace().trace(
+					GitTraceLocation.HISTORYVIEW.getLocation(),
+					"Scheduling GenerateHistoryJob"); //$NON-NLS-1$
+		schedule(job);
 	}
 
 	private IWorkbenchPartSite getPartSite() {
@@ -1821,6 +1842,7 @@ public class GitHistoryPage extends HistoryPage implements RefsChangedListener {
 	/**
 	 * {@link RevWalk#markStart(RevCommit)} all refs with given prefix to mark
 	 * start of graph traversal using currentWalker
+	 * @param walk the walk to be configured
 	 *
 	 * @param prefix
 	 *            prefix of refs to be marked
@@ -1828,54 +1850,50 @@ public class GitHistoryPage extends HistoryPage implements RefsChangedListener {
 	 * @throws MissingObjectException
 	 * @throws IncorrectObjectTypeException
 	 */
-	private void markStartAllRefs(String prefix) throws IOException,
+	private void markStartAllRefs(RevWalk walk, String prefix) throws IOException,
 			MissingObjectException, IncorrectObjectTypeException {
 		for (Entry<String, Ref> refEntry : input.getRepository()
 				.getRefDatabase().getRefs(prefix).entrySet()) {
 			Ref ref = refEntry.getValue();
 			if (ref.isSymbolic())
 				continue;
-			markStartRef(ref);
+			markStartRef(walk, ref);
 		}
 	}
 
-	private void markStartAdditionalRefs() throws IOException {
+	private void markStartAdditionalRefs(RevWalk walk) throws IOException {
 		List<Ref> additionalRefs = input.getRepository().getRefDatabase()
 				.getAdditionalRefs();
 		for (Ref ref : additionalRefs)
-			markStartRef(ref);
+			markStartRef(walk, ref);
 	}
 
-	private void markStartRef(Ref ref) throws MissingObjectException,
+	private void markStartRef(RevWalk walk, Ref ref) throws MissingObjectException,
 			IOException, IncorrectObjectTypeException {
-		Object refTarget = currentWalk.parseAny(ref.getLeaf().getObjectId());
+		Object refTarget = walk.parseAny(ref.getLeaf().getObjectId());
 		if (refTarget instanceof RevCommit)
-			currentWalk.markStart((RevCommit) refTarget);
+			walk.markStart((RevCommit) refTarget);
 	}
 
-	private void markUninteresting(String prefix) throws IOException,
+	private void markUninteresting(RevWalk walk, String prefix) throws IOException,
 			MissingObjectException, IncorrectObjectTypeException {
 		for (Entry<String, Ref> refEntry : input.getRepository()
 				.getRefDatabase().getRefs(prefix).entrySet()) {
 			Ref ref = refEntry.getValue();
 			if (ref.isSymbolic())
 				continue;
-			Object refTarget = currentWalk
+			Object refTarget = walk
 					.parseAny(ref.getLeaf().getObjectId());
 			if (refTarget instanceof RevCommit)
-				currentWalk.markUninteresting((RevCommit) refTarget);
+				walk.markUninteresting((RevCommit) refTarget);
 		}
 	}
 
-	private void cancelRefreshJob() {
-		if (job != null && job.getState() != Job.NONE) {
-			job.cancel();
-			try {
-				job.join();
-			} catch (InterruptedException e) {
-				cancelRefreshJob();
-				return;
-			}
+	private void releaseGenerateHistoryJob() {
+		if (job != null) {
+			if (job.getState() != Job.NONE)
+				job.cancel();
+			job.release();
 			job = null;
 		}
 	}
@@ -1883,4 +1901,16 @@ public class GitHistoryPage extends HistoryPage implements RefsChangedListener {
 	private boolean isShowingRelativeDates() {
 		return Activator.getDefault().getPreferenceStore().getBoolean(UIPreferences.RESOURCEHISTORY_SHOW_RELATIVE_DATE);
 	}
+
+	private boolean isShowingEmailAddresses() {
+		return Activator.getDefault().getPreferenceStore().getBoolean(UIPreferences.RESOURCEHISTORY_SHOW_EMAIL_ADDRESSES);
+	}
+
+	public boolean contains(ISchedulingRule rule) {
+		return this == rule;
+	}
+
+	public boolean isConflicting(ISchedulingRule rule) {
+		return this == rule;
+	}
 }
