@@ -168,8 +168,8 @@ public class PushWizard extends Wizard {
 			resultToCompare = confirmPage.getConfirmedResult();
 		else
 			resultToCompare = null;
-		final Job job = new PushJob(operation, resultToCompare,
-				getDestinationString());
+		final Job job = new PushJob(localDb, operation, resultToCompare,
+				getDestinationString(repoPage.getSelection()));
 
 		job.setUser(true);
 		job.schedule();
@@ -183,7 +183,7 @@ public class PushWizard extends Wizard {
 		final IWizardPage currentPage = getContainer().getCurrentPage();
 		if (currentPage == repoPage || currentPage == null)
 			return UIText.PushWizard_windowTitleDefault;
-		final String destination = getDestinationString();
+		final String destination = getDestinationString(repoPage.getSelection());
 		return NLS.bind(UIText.PushWizard_windowTitleWithDestination,
 				destination);
 	}
@@ -262,8 +262,7 @@ public class PushWizard extends Wizard {
 		}
 	}
 
-	private String getDestinationString() {
-		final RepositorySelection repoSelection = repoPage.getSelection();
+	static String getDestinationString(RepositorySelection repoSelection) {
 		final String destination;
 		if (repoSelection.isConfigSelected())
 			destination = repoSelection.getConfigName();
@@ -272,14 +271,16 @@ public class PushWizard extends Wizard {
 		return destination;
 	}
 
-	private class PushJob extends Job {
+	static class PushJob extends Job {
 		private final PushOperation operation;
 
 		private final PushOperationResult resultToCompare;
 
 		private final String destinationString;
 
-		public PushJob(final PushOperation operation,
+		private Repository localDb;
+
+		public PushJob(final Repository localDb, final PushOperation operation,
 				final PushOperationResult resultToCompare,
 				final String destinationString) {
 			super(NLS.bind(UIText.PushWizard_jobName, getURIsString(operation
@@ -287,6 +288,7 @@ public class PushWizard extends Wizard {
 			this.operation = operation;
 			this.resultToCompare = resultToCompare;
 			this.destinationString = destinationString;
+			this.localDb = localDb;
 		}
 
 		@Override
