@@ -41,11 +41,11 @@ import org.eclipse.egit.ui.UIIcons;
 import org.eclipse.egit.ui.UIText;
 import org.eclipse.egit.ui.UIUtils;
 import org.eclipse.egit.ui.internal.ConfigurationChecker;
-import org.eclipse.egit.ui.internal.repository.tree.StashedCommitNode;
 import org.eclipse.egit.ui.internal.repository.tree.FileNode;
 import org.eclipse.egit.ui.internal.repository.tree.RefNode;
 import org.eclipse.egit.ui.internal.repository.tree.RepositoryTreeNode;
 import org.eclipse.egit.ui.internal.repository.tree.RepositoryTreeNodeType;
+import org.eclipse.egit.ui.internal.repository.tree.StashedCommitNode;
 import org.eclipse.egit.ui.internal.repository.tree.TagNode;
 import org.eclipse.egit.ui.internal.trace.GitTraceLocation;
 import org.eclipse.jface.action.MenuManager;
@@ -75,6 +75,7 @@ import org.eclipse.swt.custom.StackLayout;
 import org.eclipse.swt.events.DisposeEvent;
 import org.eclipse.swt.events.DisposeListener;
 import org.eclipse.swt.graphics.Color;
+import org.eclipse.swt.graphics.Image;
 import org.eclipse.swt.widgets.Composite;
 import org.eclipse.swt.widgets.Display;
 import org.eclipse.swt.widgets.Label;
@@ -151,11 +152,11 @@ public class RepositoriesView extends CommonNavigator {
 
 	private StackLayout layout;
 
-	private long lastInputChange = 0L;
+	private volatile long lastInputChange = 0L;
 
-	private long lastRepositoryChange = 0L;
+	private volatile long lastRepositoryChange = 0L;
 
-	private long lastInputUpdate = -1L;
+	private volatile long lastInputUpdate = -1L;
 
 	private boolean reactOnSelection = false;
 
@@ -213,9 +214,8 @@ public class RepositoriesView extends CommonNavigator {
 						reactOnSelection(new StructuredSelection(
 								((IFileEditorInput) input).getFile()));
 
-				} else {
+				} else
 					reactOnSelection(selection);
-				}
 			}
 		};
 	}
@@ -260,7 +260,9 @@ public class RepositoriesView extends CommonNavigator {
 				.getDisplay());
 
 		Label addLabel = new Label(optionsArea, SWT.NONE);
-		addLabel.setImage(UIIcons.CREATE_REPOSITORY.createImage());
+		Image addImage = UIIcons.CREATE_REPOSITORY.createImage();
+		UIUtils.hookDisposal(addLabel, addImage);
+		addLabel.setImage(addImage);
 		Hyperlink addLink = toolkit.createHyperlink(optionsArea,
 				UIText.RepositoriesView_linkAdd, SWT.WRAP);
 		addLink.setForeground(linkColor);
@@ -276,7 +278,9 @@ public class RepositoriesView extends CommonNavigator {
 				.grab(true, false).applyTo(addLink);
 
 		Label cloneLabel = new Label(optionsArea, SWT.NONE);
-		cloneLabel.setImage(UIIcons.CLONEGIT.createImage());
+		Image cloneImage = UIIcons.CLONEGIT.createImage();
+		UIUtils.hookDisposal(cloneLabel, cloneImage);
+		cloneLabel.setImage(cloneImage);
 		Hyperlink cloneLink = toolkit.createHyperlink(optionsArea,
 				UIText.RepositoriesView_linkClone, SWT.WRAP);
 		cloneLink.setForeground(linkColor);
@@ -292,7 +296,9 @@ public class RepositoriesView extends CommonNavigator {
 				.grab(true, false).applyTo(cloneLink);
 
 		Label createLabel = new Label(optionsArea, SWT.NONE);
-		createLabel.setImage(UIIcons.NEW_REPOSITORY.createImage());
+		Image createImage = UIIcons.NEW_REPOSITORY.createImage();
+		UIUtils.hookDisposal(createLabel, createImage);
+		createLabel.setImage(createImage);
 		Hyperlink createLink = toolkit.createHyperlink(optionsArea,
 				UIText.RepositoriesView_linkCreate, SWT.WRAP);
 		createLink.setForeground(linkColor);
@@ -368,9 +374,8 @@ public class RepositoriesView extends CommonNavigator {
 				TreeSelection sel = (TreeSelection) event.getSelection();
 				RepositoryTreeNode element = (RepositoryTreeNode) sel
 						.getFirstElement();
-				if (element instanceof RefNode || element instanceof TagNode) {
+				if (element instanceof RefNode || element instanceof TagNode)
 					executeOpenCommand();
-				}
 			}
 		});
 		// handle open event for the working directory
@@ -485,20 +490,21 @@ public class RepositoriesView extends CommonNavigator {
 			RepositoryMapping mapping = RepositoryMapping.getMapping(project);
 			if (mapping == null)
 				return;
+			String repoPath = mapping.getRepoRelativePath(resource);
+			if( repoPath == null)
+				return;
 
 			boolean added = repositoryUtil.addConfiguredRepository(mapping
 					.getRepository().getDirectory());
-			if (added) {
+			if (added)
 				scheduleRefresh(0);
-			}
 
-			if (this.scheduledJob != null) {
+			if (this.scheduledJob != null)
 				try {
 					this.scheduledJob.join();
 				} catch (InterruptedException e) {
 					Activator.handleError(e.getMessage(), e, false);
 				}
-			}
 
 			RepositoryTreeNode currentNode = null;
 			ITreeContentProvider cp = (ITreeContentProvider) getCommonViewer()
@@ -519,9 +525,9 @@ public class RepositoriesView extends CommonNavigator {
 				}
 			}
 
-			IPath relPath = new Path(mapping.getRepoRelativePath(resource));
+			IPath relPath = new Path(repoPath);
 
-			for (String segment : relPath.segments()) {
+			for (String segment : relPath.segments())
 				for (Object child : cp.getChildren(currentNode)) {
 					RepositoryTreeNode<File> childNode = (RepositoryTreeNode<File>) child;
 					if (childNode.getObject().getName().equals(segment)) {
@@ -529,7 +535,6 @@ public class RepositoriesView extends CommonNavigator {
 						break;
 					}
 				}
-			}
 
 			final RepositoryTreeNode selNode = currentNode;
 
@@ -606,9 +611,12 @@ public class RepositoriesView extends CommonNavigator {
 				if (needsNewInput)
 					initRepositoriesAndListeners();
 
-				Display.getDefault().asyncExec(new Runnable() {
+				if (!UIUtils.isUsable(tv))
+					return Status.CANCEL_STATUS;
+				PlatformUI.getWorkbench().getDisplay()
+						.asyncExec(new Runnable() {
 					public void run() {
-						if (tv.getTree().isDisposed())
+						if (!UIUtils.isUsable(tv))
 							return;
 						long start = 0;
 						boolean traceActive = GitTraceLocation.REPOSITORIESVIEW
@@ -627,10 +635,10 @@ public class RepositoriesView extends CommonNavigator {
 						IStructuredSelection sel = (IStructuredSelection) tv
 								.getSelection();
 
-						if (needsNewInput) {
+						if (needsNewInput)
 							tv.setInput(ResourcesPlugin.getWorkspace()
 									.getRoot());
-						} else
+						else
 							tv.refresh(true);
 						tv.setExpandedElements(expanded);
 
