@@ -15,6 +15,7 @@ import java.io.File;
 import org.eclipse.core.runtime.PlatformObject;
 import org.eclipse.jgit.lib.Ref;
 import org.eclipse.jgit.lib.Repository;
+import org.eclipse.jgit.revwalk.RevCommit;
 
 /**
  * A node in the Git Repositories view tree
@@ -132,6 +133,10 @@ public abstract class RepositoryTreeNode<T> extends PlatformObject implements Co
 			// fall through
 		case ADDITIONALREFS:
 			// fall through
+		case SUBMODULES:
+			// fall through
+		case STASH:
+			// fall through
 		case WORKINGDIR:
 			result = prime
 					* result
@@ -166,6 +171,8 @@ public abstract class RepositoryTreeNode<T> extends PlatformObject implements Co
 			// fall through
 		case BRANCHHIERARCHY:
 			// fall through
+		case STASHED_COMMIT:
+			// fall through
 		case ERROR:
 			result = prime * result
 					+ ((myObject == null) ? 0 : myObject.hashCode());
@@ -246,6 +253,10 @@ public abstract class RepositoryTreeNode<T> extends PlatformObject implements Co
 			// fall through
 		case ERROR:
 			// fall through
+		case SUBMODULES:
+			// fall through
+		case STASH:
+			// fall through
 		case WORKINGDIR:
 			return 0;
 
@@ -261,6 +272,9 @@ public abstract class RepositoryTreeNode<T> extends PlatformObject implements Co
 		case FOLDER:
 			return ((File) myObject).getName().compareTo(
 					((File) otherNode.getObject()).getName());
+		case STASHED_COMMIT:
+			return ((RevCommit) myObject).compareTo(((RevCommit) otherNode
+					.getObject()));
 		case TAG:
 			// fall through
 		case ADDITIONALREF:
@@ -308,6 +322,10 @@ public abstract class RepositoryTreeNode<T> extends PlatformObject implements Co
 			// fall through
 		case ADDITIONALREFS:
 			// fall through
+		case SUBMODULES:
+			// fall through
+		case STASH:
+			// fall through
 		case WORKINGDIR:
 			return ((Repository) myObject).getDirectory().equals(
 					((Repository) otherObject).getDirectory());
@@ -333,6 +351,8 @@ public abstract class RepositoryTreeNode<T> extends PlatformObject implements Co
 			// fall through
 		case BRANCHHIERARCHY:
 			// fall through
+		case STASHED_COMMIT:
+			// fall through
 		case TAGS:
 			return myObject.equals(otherObject);
 		}
