@@ -14,9 +14,10 @@ import java.io.File;
 
 import org.eclipse.core.commands.ExecutionEvent;
 import org.eclipse.core.commands.ExecutionException;
-import org.eclipse.egit.ui.internal.repository.RepositorySearchDialog;
+import org.eclipse.egit.ui.internal.repository.RepositorySearchWizard;
 import org.eclipse.egit.ui.internal.repository.tree.RepositoryTreeNode;
 import org.eclipse.jface.window.Window;
+import org.eclipse.jface.wizard.WizardDialog;
 
 /**
  * "Adds" repositories
@@ -24,10 +25,11 @@ import org.eclipse.jface.window.Window;
 public class AddCommand extends
 		RepositoriesViewCommandHandler<RepositoryTreeNode> {
 	public Object execute(ExecutionEvent event) throws ExecutionException {
-		RepositorySearchDialog sd = new RepositorySearchDialog(getShell(event),
+		RepositorySearchWizard wizard = new RepositorySearchWizard(
 				util.getConfiguredRepositories());
-		if (sd.open() == Window.OK)
-			for (String dir : sd.getDirectories())
+		WizardDialog dialog = new WizardDialog(getShell(event), wizard);
+		if (dialog.open() == Window.OK)
+			for (String dir : wizard.getDirectories())
 				util.addConfiguredRepository(new File(dir));
 		return null;
 	}
