@@ -79,6 +79,8 @@ import org.eclipse.swt.widgets.Text;
 import org.eclipse.swt.widgets.Tree;
 import org.eclipse.swt.widgets.TreeColumn;
 import org.eclipse.swt.widgets.TreeItem;
+import org.eclipse.ui.PlatformUI;
+import org.eclipse.ui.ide.IDE.SharedImages;
 
 /**
  * Wizard page for connecting projects to Git repositories.
@@ -97,7 +99,7 @@ class ExistingOrNewPage extends WizardPage {
 
 	private IPath minumumPath;
 
-	private Text dotGitSegment;
+	private Label dotGitSegment;
 
 	private Composite externalComposite;
 
@@ -216,13 +218,16 @@ class ExistingOrNewPage extends WizardPage {
 			public void widgetSelected(SelectionEvent e) {
 				DirectoryDialog dlg = new DirectoryDialog(getShell());
 				dlg.setFilterPath(selectedRepository.getWorkTree().getPath());
-				setRelativePath(dlg.open());
-				updateControls();
+				String directory = dlg.open();
+				if (directory != null) {
+					setRelativePath(directory);
+					updateControls();
+				}
 			}
 		});
 
 		Table projectMoveTable = new Table(externalComposite, SWT.MULTI
-				| SWT.FULL_SELECTION | SWT.CHECK);
+				| SWT.FULL_SELECTION | SWT.CHECK | SWT.BORDER);
 		projectMoveViewer = new CheckboxTableViewer(projectMoveTable);
 		GridDataFactory.fillDefaults().span(3, 1).grab(true, true)
 				.applyTo(projectMoveTable);
@@ -230,7 +235,7 @@ class ExistingOrNewPage extends WizardPage {
 		TableColumn tc;
 		tc = new TableColumn(projectMoveTable, SWT.NONE);
 		tc.setText(UIText.ExistingOrNewPage_ProjectNameColumnHeader);
-		tc.setWidth(150);
+		tc.setWidth(100);
 
 		tc = new TableColumn(projectMoveTable, SWT.NONE);
 		tc.setText(UIText.ExistingOrNewPage_CurrentLocationColumnHeader);
@@ -304,11 +309,12 @@ class ExistingOrNewPage extends WizardPage {
 		c1.setText(UIText.ExistingOrNewPage_HeaderProject);
 		c1.setWidth(100);
 		TreeColumn c2 = new TreeColumn(tree, SWT.NONE);
-		c2.setText(UIText.ExistingOrNewPage_HeaderPath);
+		c2.setText(UIText.ExistingOrNewPage_HeaderLocation);
 		c2.setWidth(400);
 		TreeColumn c3 = new TreeColumn(tree, SWT.NONE);
 		c3.setText(UIText.ExistingOrNewPage_HeaderRepository);
 		c3.setWidth(200);
+		boolean allProjectsInExistingRepos = true;
 		for (IProject project : myWizard.projects) {
 			RepositoryFinder repositoryFinder = new RepositoryFinder(project);
 			try {
@@ -319,14 +325,15 @@ class ExistingOrNewPage extends WizardPage {
 				if (m == null) {
 					// no mapping found, enable repository creation
 					TreeItem treeItem = new TreeItem(tree, SWT.NONE);
-					treeItem.setText(0, project.getName());
+					updateProjectTreeItem(treeItem, project);
 					treeItem.setText(1, project.getLocation().toOSString());
 					treeItem.setText(2, ""); //$NON-NLS-1$
 					treeItem.setData(new ProjectAndRepo(project, "")); //$NON-NLS-1$
+					allProjectsInExistingRepos = false;
 				} else if (!mi.hasNext()) {
 					// exactly one mapping found
 					TreeItem treeItem = new TreeItem(tree, SWT.NONE);
-					treeItem.setText(0, project.getName());
+					updateProjectTreeItem(treeItem, project);
 					treeItem.setText(1, project.getLocation().toOSString());
 					fillTreeItemWithGitDirectory(m, treeItem, false);
 					treeItem.setData(new ProjectAndRepo(project, treeItem
@@ -336,24 +343,25 @@ class ExistingOrNewPage extends WizardPage {
 
 				else {
 					TreeItem treeItem = new TreeItem(tree, SWT.NONE);
-					treeItem.setText(0, project.getName());
+					updateProjectTreeItem(treeItem, project);
 					treeItem.setText(1, project.getLocation().toOSString());
 					treeItem.setData(new ProjectAndRepo(null, null));
 
 					TreeItem treeItem2 = new TreeItem(treeItem, SWT.NONE);
-					treeItem2.setText(0, project.getName());
+					updateProjectTreeItem(treeItem2, project);
 					fillTreeItemWithGitDirectory(m, treeItem2, true);
 					treeItem2.setData(new ProjectAndRepo(project, treeItem2
 							.getText(2)));
 					while (mi.hasNext()) { // fill in additional mappings
 						m = mi.next();
 						treeItem2 = new TreeItem(treeItem, SWT.NONE);
-						treeItem2.setText(0, project.getName());
+						updateProjectTreeItem(treeItem2, project);
 						fillTreeItemWithGitDirectory(m, treeItem2, true);
 						treeItem2.setData(new ProjectAndRepo(m.getContainer()
 								.getProject(), treeItem2.getText(2)));
 					}
 					treeItem.setExpanded(true);
+					allProjectsInExistingRepos = false;
 				}
 			} catch (CoreException e) {
 				TreeItem treeItem2 = new TreeItem(tree, SWT.BOLD | SWT.ITALIC);
@@ -422,9 +430,8 @@ class ExistingOrNewPage extends WizardPage {
 						.segmentCount());
 			}
 		});
-		dotGitSegment = new Text(parentRepoComposite, SWT.NONE);
+		dotGitSegment = new Label(parentRepoComposite, SWT.NONE);
 		dotGitSegment.setEnabled(false);
-		dotGitSegment.setEditable(false);
 		dotGitSegment.setText(File.separatorChar + Constants.DOT_GIT);
 		dotGitSegment.setLayoutData(GridDataFactory.fillDefaults()
 				.align(SWT.LEFT, SWT.CENTER).create());
@@ -438,6 +445,19 @@ class ExistingOrNewPage extends WizardPage {
 
 		Dialog.applyDialogFont(main);
 		setControl(main);
+
+		if (allProjectsInExistingRepos) {
+			internalMode = true;
+			internalModeButton.setSelection(true);
+			updateControls();
+		}
+	}
+
+	private void updateProjectTreeItem(TreeItem item, IProject project) {
+		item.setImage(0,
+				PlatformUI.getWorkbench().getSharedImages()
+						.getImage(SharedImages.IMG_OBJ_PROJECT));
+		item.setText(0, project.getName());
 	}
 
 	protected void setRelativePath(String directory) {
